precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float ratio;
uniform mat4 type;
uniform mat4 value;
float valueuse;
vec2 pos;

//现在把锐化，阴影跟颗粒从里面剥离出去了 picadjust.glsl
vec3 rgbToHsv(vec3 c) {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsvToRgb(vec3 c) {//convertHsvToRgb
                       vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
                       vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
                       return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}


//亮度
//https://juejin.cn/post/7057793339164196900
vec4 getbrightness(vec4 color) {
    float value = valueuse / 500.0;
    color = vec4((color.rgb + vec3(value)), color.a);
    //   color = vec4(color.rgb * pow(2.0, value), color.a);// rgb * 2^效果值
    return color;
}
vec4 getExposure2(vec4 textureColor) {
    float exposure = valueuse / 250.0;
    return vec4(textureColor.rgb * pow(2.0, exposure), textureColor.a);// rgb * 2^效果值
}


//可能是曝光度
//https://segmentfault.com/a/1190000037668990
vec4 getExposure(vec4 textureColor) {
    float exposure = valueuse / 100.0;
    return vec4(textureColor.rgb * pow(2.0, exposure), textureColor.a);// rgb * 2^效果值
}
//对比度
vec4 getcontrast(vec4 color) {
    float progress = 1.0 + (valueuse / 200.0);
    return vec4(((color.rgb - vec3(0.5)) * progress + vec3(0.5)), color.a);
}
//饱和度
vec4 getsaturation(vec4 color) {
    vec3 luminanceWeighting = vec3(0.2125, 0.7154, 0.0721);
    //    范围0-2;
    float saturation = (valueuse / 50.0) + 1.0;
    vec3 greyScaleColor = vec3(dot(color.rgb, luminanceWeighting));
    vec4 rtn = vec4(mix(greyScaleColor, color.rgb, saturation), color.a);
    return rtn;
}

//高亮 加亮度版
vec4 getHighLight(vec4 color) {
    float progress = valueuse / 200.0;
    vec3 hsl = rgbToHsv(color.xyz);
    float value = hsl.z;
    value += progress;
    value = max(value, 0.0);
    value = min(value, 1.0);
    hsl.z = value;
    vec3 use = hsvToRgb(hsl);
    return vec4(use, color.a);
}

vec4 test(vec4 color) {
    float b = valueuse / 800.0;
    vec3 use = ((color.rgb * 2.0 - (1.0 - b)) + (1.0 + b)) / 2.0;
    return vec4(use, color.a);
}



//高亮 GPUImageHighlightFilter
vec3 luminanceWeighting = vec3(0.32);
vec4 getHighLight2(vec4 source) {
    bool change = valueuse > 0.0;
    float highlights = 1.0 - abs(valueuse / 200.1);
    float luminance = dot(source.rgb, luminanceWeighting);
    float highlight = clamp((1.0 - (pow(1.0 - luminance, 1.0 / (2.0 - highlights)) + (-0.1) * pow(1.0 - luminance, 2.0 / (2.0 - highlights)))) - luminance, -1.0, 0.0);
    vec3 result = ((luminance + highlight)) * ((source.rgb) / (luminance));
    vec3 cha = result.rgb - source.rgb;
    float xishu = 2.0;
    if (change) {
        result = source.rgb - cha * xishu;
    } else {
        result = source.rgb + cha * xishu;
    }

    vec4 rtn = vec4(result.rgb, source.a);
    return rtn;
}
// 用的collage的 FilterTask GPUImageHighlightFilter
//阴影
vec4 getShade(vec4 source) {
    bool change = valueuse < 0.0;
    float shadows = abs(valueuse) / 100.0;
    float luminance = dot(source.rgb, luminanceWeighting);
    float shadow = clamp((pow(luminance, 1.0 / (shadows + 1.0)) + (-0.05) * pow(luminance, 2.0 / (shadows + 1.0))) - luminance, 0.0, 1.0);
    vec3 result = (luminance + shadow) * source.rgb / luminance;
    if (change) {
        result = source.rgb - (result.rgb - source.rgb);
    }
    vec4 rtn = vec4(result.rgb, source.a);
    return rtn;
}

//高光阴影的原始算法
//vec4 getHighLightAndShadow(vec4 source, float highlights, float shadows) {
//    highlights = highlights / 50.0 + 1.0;
//    shadows /= 50.0;
//    vec3 luminanceWeighting = vec3(0.3);
//
//    float luminance = dot(source.rgb, luminanceWeighting);
//    float shadow = clamp((pow(luminance, 1.0 / (shadows + 1.0)) + (-0.76) * pow(luminance, 2.0 / (shadows + 1.0))) - luminance, 0.0, 1.0);
//    float highlight = clamp((1.0 - (pow(1.0 - luminance, 1.0 / (2.0 - highlights)) + (-0.8) * pow(1.0 - luminance, 2.0 / (2.0 - highlights)))) - luminance, -1.0, 0.0);
//    vec3 result = ((luminance + shadow + highlight)) * ((source.rgb) / (luminance));
//
//    vec4 rtn = vec4(result.rgb, source.a);
//    return rtn;
//}

//色温色调一起处理
vec4 getWhiteBalance(vec4 source, float temperaturevalue, float tintvalue) {
    //    色温参数
    float temperature = temperaturevalue / 800.0;
    //    控制色调
    float tint = tintvalue / 150.0;
    vec3 warmFilter = vec3(3, 2.3, 0);
    mat3 RGBtoYIQ = mat3(0.299, 0.587, 0.114, 0.596, -0.274, -0.322, 0.212, -0.523, 0.311);
    mat3 YIQtoRGB = mat3(1.0, 0.956, 0.621, 1.0, -0.272, -0.647, 1.0, -1.105, 1.702);

    mediump vec3 yiq = RGBtoYIQ * source.rgb;
    yiq.b = clamp(yiq.b + tint * 0.5226 * 0.1, -0.5226, 0.5226);
    lowp vec3 rgb = YIQtoRGB * yiq;
    lowp vec3 processed = vec3(
        (rgb.r < 0.5 ? (2.0 * rgb.r * warmFilter.r) : (1.0 - 2.0 * (1.0 - rgb.r) * (1.0 - warmFilter.r))),
        (rgb.g < 0.5 ? (2.0 * rgb.g * warmFilter.g) : (1.0 - 2.0 * (1.0 - rgb.g) * (1.0 - warmFilter.g))),
        (rgb.b < 0.5 ? (2.0 * rgb.b * warmFilter.b) : (1.0 - 2.0 * (1.0 - rgb.b) * (1.0 - warmFilter.b))));
    vec4 rtn = vec4(mix(rgb, processed, temperature), source.a);
    return rtn;
}

//褪色
vec4 getFade(vec4 source) {
    vec3 c = vec3(0.5);
    float progress = 1.0 - valueuse / 100.0 / 2.0;
    vec3 cha = source.rgb - c;
    cha *= progress;
    cha += c;
    return vec4(cha, source.a);
}

void main() {
    pos = textureCoordinate / 1.0;
    vec4 color = texture2D(inputImageTexture, pos);

    if (type[0][0] > 0.0) {
        valueuse = value[0][0];
        color = getbrightness(color);
        color = max(vec4(0.0), color);
        color = min(vec4(1.0), color);


        color = getExposure2(color);
        color = max(vec4(0.0), color);
        color = min(vec4(1.0), color);
    }
    if (type[0][1] > 0.0) {
        valueuse = value[0][1];
        color = getcontrast(color);
        color = max(vec4(0.0), color);
        color = min(vec4(1.0), color);
    }
    if (type[0][2] > 0.0) {
        valueuse = value[0][2];
        color = getsaturation(color);
        color = max(vec4(0.0), color);
        color = min(vec4(1.0), color);
    }
    if (type[1][0] > 0.0) {
        valueuse = value[1][0];
        color = getHighLight2(color);
        color = max(vec4(0.0), color);
        color = min(vec4(1.0), color);
    }
    if (type[1][1] > 0.0) {
        valueuse = value[1][1];
        color = getShade(color);
        color = max(vec4(0.0), color);
        color = min(vec4(1.0), color);
    }
    if (type[1][2] > 0.0 || type[1][3] > 0.0) {
        color = getWhiteBalance(color, value[1][2], value[1][3]);
        color = max(vec4(0.0), color);
        color = min(vec4(1.0), color);

    }
    if (type[2][0] > 0.0) {
        valueuse = value[2][0];
        color = getFade(color);
        color = max(vec4(0.0), color);
        color = min(vec4(1.0), color);
    }
    if (type[2][3] > 0.0) {
        valueuse = value[2][3];
        color = getExposure(color);
        color = max(vec4(0.0), color);
        color = min(vec4(1.0), color);
    }
    gl_FragColor = color;
}
