precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float ratio1;
uniform mat4 type;
uniform mat4 value;
highp vec2 center = vec2(0.5);
float valueuse;
vec2 pos;
vec2 inputSize = vec2(1080.0);

//锐化
//https://cloud.tencent.com/developer/article/2379771
vec4 getSharpening(vec4 color) {
    float step = (valueuse / 100.0) * 0.5;
    vec2 uvOffsets[9];
    float dx = step / inputSize.x;
    float dy = step / inputSize.y;
    float i2 = 0.0;
    float j2 = 0.0;
    for (int i = 0; i < 3; i++, i2++) {
        for (int j = 0; j < 3; j++, j2++) {
            uvOffsets[((i * 3) + j)].x = (-1.0 + i2) * dx;
            uvOffsets[((i * 3) + j)].y = (-1.0 + j2) * dy;
        }
    }
    vec4 sampleCols[9];
    for (int i = 0; i < 9; i++)
    {
        // 采样邻域的网格
        sampleCols[i].rgba = texture2D(inputImageTexture, pos + uvOffsets[i]);
    }
    // 锐化卷积核 3x3
    // -1 -1 -1
    // -1  9 -1
    // -1 -1 -1

    vec4 result = 9.0 * sampleCols[4].rgba;
    for (int i = 0; i < 9; i++)
    {
        if (i != 4)
        result -= sampleCols[i];
    }
    vec4 orgResult = color;
    return result;
}




//暗角
vec4 getVignetting(vec4 source) {
    vec3 usecolor = valueuse > 0.0 ? vec3(0.0) : vec3(1.0);
    //   点与中心的距离 0->1
    float length_center = distance(pos, center) * 2.0 / 1.42;
    //   进度 0->1
    float progress = abs(valueuse) / 50.0;

    //    把进度转化成一个圆的长度
    float minl = 1.0 - progress * 0.7;
    float mixvalue = length_center;
    float l = 0.5;
    if (mixvalue < minl) {
        mixvalue = 0.0;
    } else if (mixvalue > minl + l) {
        mixvalue = 1.0;
    } else {
        mixvalue = (mixvalue - minl) / l;
        mixvalue *= mixvalue;
    }
    //    mixvalue = mix(minl, minl + l, mixvalue);

    vec3 color = mix(source.rgb, usecolor, mixvalue);
    return vec4(color, source.a);
}

float Rand(vec2 v) {
    return fract(sin(dot(v.xy, vec2(12.9898, 78.233))) * 43758.5453);
}
//颗粒
vec4 getgrain(vec4 source) {
    //    0-0.5
    //    float pro = valueuse / 200.0;
    //    //    0.25-0.75
    //    float random = (0.25 + Rand(pos) / 2.0);
    //    vec3 color = mix(source.rgb,vec3(random), pro);

    float pro = valueuse / 100.0 / 4.0;
    //    1-1.5;
    float random = Rand(pos) + 0.5;
    random = (random - 1.0) * pro + 1.0;
    vec3 color = source.rgb * random;
    return vec4(color, source.a);
}



void main() {
    pos = textureCoordinate / 1.0;
    vec4 color = texture2D(inputImageTexture, pos);
    if (type[0][3] > 0.0) {
        if (ratio1 > 1.0) {
            inputSize.x *= ratio1;
        } else {
            inputSize.y /= ratio1;
        }
        valueuse = value[0][3];
        color = getSharpening(color);
    }
    if (type[2][1] > 0.0) {
        color = max(vec4(0.0), color);
        color = min(vec4(1.0), color);
        valueuse = value[2][1];
        color = getVignetting(color);
    }
    if (type[2][2] > 0.0) {
        color = max(vec4(0.0), color);
        color = min(vec4(1.0), color);
        valueuse = value[2][2];
        color = getgrain(color);
    }
    gl_FragColor = color;
}
