precision highp float;
varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;
uniform sampler2D inputImageTexture4;
//加载模式
uniform ivec4 inBlendMode;
//纹理加载顺序
uniform ivec4 inBlendTex;
//纹理与原图的混合程度
uniform float mixvalue;
//有些纹理要横变竖颠倒
uniform int lvroate;
//screen 改变色相
uniform float huevalue;

//screen 改变色相
uniform float lutvalue;

vec3 rgbToHsv(vec3 c) {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsvToRgb(vec3 c) {//convertHsvToRgb
                       vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
                       vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
                       return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

vec4 changecolorhue(vec4 color) {
    vec3 hsv = rgbToHsv(color.rgb);
    hsv.r += huevalue;
    hsv.r = fract(hsv.r);
    color.rgb = hsvToRgb(hsv);
    return color;
}

//求随机数
//float Rand(vec2 v) {
//    return fract(sin(dot(v.xy, vec2(12.9898, 78.233))) * 43758.5453);
//}

/// Screen 滤色 1   C=255-(A反相×B反相)/255
vec4 getscreencolor(vec4 base, vec4 overlay) {
    vec4 one = vec4(1.0);
    overlay = changecolorhue(overlay);
    vec4 rtn = one - (one - base) * (one - overlay);
    return rtn;
}
/// 透明度混合 2
vec4 getalphacolor(vec4 base, vec4 overlay) {
    vec4 one = vec4(1.0);
    overlay = changecolorhue(overlay);
    vec4 rtn = base * (one - overlay.a) + overlay * overlay.a;
    return vec4(rtn.rgb, 1.0);
}
//  绿幕 3
vec4 getgreenscreencolor(vec4 base, vec4 overlay) {
    if (overlay.g - overlay.r >= 0.1 && overlay.g - overlay.b >= 0.1) {
        overlay = vec4(overlay.r, (overlay.r + overlay.b) / 2.0, overlay.b, 1.0 - overlay.g);
    } else {
        overlay = vec4(overlay.r, overlay.g, overlay.b, overlay.a);
    }
    vec4 rtn = mix(base, overlay, overlay.a);
    return rtn;
}
//  lut 4
vec4 getlut(vec4 base, sampler2D overlay) {
    highp float blueColor = base.b * 63.0;
    highp vec2 quad1;
    quad1.y = floor(floor(blueColor) / 8.0);
    quad1.x = floor(blueColor) - (quad1.y * 8.0);

    highp vec2 quad2;
    quad2.y = floor(ceil(blueColor) / 8.0);
    quad2.x = ceil(blueColor) - (quad2.y * 8.0);

    highp vec2 texPos1;
    texPos1.x = (quad1.x * 0.125) + 0.5 / 512.0 + ((0.125 - 1.0 / 512.0) * base.r);
    texPos1.y = (quad1.y * 0.125) + 0.5 / 512.0 + ((0.125 - 1.0 / 512.0) * base.g);

    highp vec2 texPos2;
    texPos2.x = (quad2.x * 0.125) + 0.5 / 512.0 + ((0.125 - 1.0 / 512.0) * base.r);
    texPos2.y = (quad2.y * 0.125) + 0.5 / 512.0 + ((0.125 - 1.0 / 512.0) * base.g);

    highp vec4 newColor1 = texture2D(overlay, texPos1);
    highp vec4 newColor2 = texture2D(overlay, texPos2);

    highp vec4 rtn = mix(newColor1, newColor2, fract(blueColor));
    //    变成lut文件颜色
    return rtn;
}
//      alphanext=5.0
vec4 getalphanext(vec4 base, vec4 overlay) {
    base.a = overlay.a;
    return base;
}
vec4 getcolor(int type) {
    lowp vec4 overlay;
    if (type == 0) {
        overlay = texture2D(inputImageTexture2, textureCoordinate);
    } else if (type == 1) {
        overlay = texture2D(inputImageTexture3, textureCoordinate);
    } else if (type == 2) {
        overlay = texture2D(inputImageTexture4, textureCoordinate);
    }
    return overlay;
}

void main()
{
    lowp vec4 base = texture2D(inputImageTexture, textureCoordinate);
    lowp vec4 baseold = base;
    bool keepalpha = false;
    if (inBlendMode.r > 0) {
        //                根据 i 替换纹理，根据inBlendMode[i]来判断用哪个
        for (int i = 0;i <= 3; i++) {
            //                    换纹理
            int type = inBlendMode[i];
            int typet = inBlendTex[i];

            if (type > 0) {
                lowp vec4 overlay = getcolor(typet);
                if (type == 1) {
                    base = getscreencolor(base, overlay);
                }
                else if (type == 2) {
                    base = getalphacolor(base, overlay);
                }
                else if (type == 3) {
                    if (lvroate == 1) {
                        overlay = getcolor(typet);
                    }
                    base = getgreenscreencolor(base, overlay);
                    //                            base=overlay;
                }
                else if (type == 4) {
                    vec4 old=base;
                    if (typet == 0) {
                        base = getlut(base, inputImageTexture2);
                    } else if (typet == 1) {
                        base = getlut(base, inputImageTexture3);
                    }else if (typet == 2) {
                        base = getlut(base, inputImageTexture4);
                    }
                    base=mix(old,base,lutvalue);
                } else if (type == 5) {
                    base = getalphanext(base, overlay);
                    keepalpha = true;
                }
            } else {
                break;
            }
        }
    }

    if (!keepalpha){
        base.a = 1.0;
        base=mix(baseold,base,mixvalue);
    }
    gl_FragColor = base;
}