/// 蒙版抠图脚本
precision mediump float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform lowp float ratio;//真实比例

// 蒙版形状类型
uniform int shape;
// 中心点位置
uniform vec2 center;
// 蒙版形状大小
uniform vec2 size;
// 边缘模糊级别
uniform float edge;
// 蒙版旋转角度
uniform float rotate;

#define PI 3.14159265359

/// 画圆或者椭圆
float doCircular(){
    // 短边基于1，放大长边，使x与y坐标单位刻度一致
    vec2 ratioNew = ratio<1.0?vec2(1.0, 1.0 / ratio):vec2(ratio, 1.0);
    // 转换当前纹理坐标到新坐标系
    vec2 textureCoordinateNew = textureCoordinate * ratioNew;
    // 转换中心点到新坐标系
    vec2 centerNew = center * ratioNew;
    // 转换宽高到新坐标系
    vec2 sizeNew = size * ratioNew;

    // 将转换后的纹理坐标绕转换后的中心点旋转，如果旋转后的纹理坐标落在椭圆内，则原纹理坐标点显示
    mat2 rotateMatrix = mat2(cos(rotate), -sin(rotate), sin(rotate), cos(rotate));
    textureCoordinateNew -= centerNew;
    textureCoordinateNew = textureCoordinateNew * rotateMatrix;
    textureCoordinateNew += centerNew;

    // 当前点到中心点的距离
    float distanceOfCenter = distance(textureCoordinateNew, centerNew);

    // 当前点相对中心点角度
    float degreeOfCenter = atan((textureCoordinateNew.y-centerNew.y), (textureCoordinateNew.x-centerNew.x));

    // 方法1
    // 当前角度下，根据椭圆公式，计算椭圆边缘点到中心点距离
    float width2 = pow(sizeNew.x/2.0, 2.0);
    float height2 = pow(sizeNew.y/2.0, 2.0);
    float lengthOfEdgePoint = width2*height2/(width2 * pow(sin(degreeOfCenter), 2.0) + height2 * pow(cos(degreeOfCenter), 2.0) );
    lengthOfEdgePoint = sqrt(lengthOfEdgePoint);


    /*
    // 方法2
    // 当前角度下，先计算离心角，计算椭圆边缘点到中心点距离
    float centrifugalAngle = tan(degreeOfCenter)*(sizeNew.x/sizeNew.y);
    centrifugalAngle = atan(centrifugalAngle);
    vec2 edgePoint = vec2(sizeNew.x/2.0 * cos(centrifugalAngle), sizeNew.y/2.0 * sin(centrifugalAngle));
    lengthOfEdgePoint = length(edgePoint);*/

    return smoothstep(lengthOfEdgePoint, lengthOfEdgePoint+edge, distanceOfCenter); //step(lengthOfEdgePoint, distanceOfCenter); //
}

// p: 像素点坐标[-1, 1]
// o: 五角星中心点坐标
// r: 五角星半径，这个半径只是一个大约值，很难精准控制
// 返回值: 当前点到五角星的边的最近距离，小于0在五角星内，大于0在五角星外
float polarStar( in vec2 p, in vec2 o, in float r )
{
    const float pi5 = 0.628318530718; // pi/5
    // const float ph2 = 3.2360679775; // 2 * phi
    const float ph2 = 3.2360679775*2.0; // 2 * phi // 黄金比率, PHI=1.618034。
    r = ph2/r;

    // float m2 = mod(atan(p.y, p.x)/pi5 + 1.0, 2.0);
    // 0.0<=atan(p.y-o.y, p.x-o.x)/pi5<=10.0
    // 0.0<=m2<2.0，且循环了5个周期
    float m2 = mod(atan(p.y-o.y, p.x-o.x)/pi5 + 1.0, 2.0);

    // return ph2 * length(p) * cos(pi5 * (m2 - 4.0 * step(1.0, m2) + 1.0)) - 1.0;
    // x = (m2 - 4.0 * step(1.0, m2) + 1.0)。当m2<1.0, 1.0 =< x < 2.0。 当m2>=1.0, -2.0 <= x <= -1.0
    // y = cos(pi5 * (m2 - 4.0 * step(1.0, m2) + 1.0))。 当m2<1.0, cos(pi5*2.0) < y < cos(pi5)。 当m2>=1.0, cos(pi5*2.0) < y < cos(pi5)。最终, cos(pi5)=0.8, cos(pi5*2.0)=0.3
    return r * distance(p, o) * cos(pi5 * (m2 - 4.0 * step(1.0, m2) + 1.0)) - 1.0;
}

// 五角星的，不支持拉伸
float doFivePointedStar(){
    // 短边基于1，放大长边，使x与y坐标单位刻度一致
    vec2 ratioNew = ratio<1.0?vec2(1.0, 1.0 / ratio):vec2(ratio, 1.0);
    // 转换当前纹理坐标到新坐标系
    vec2 textureCoordinateNew = textureCoordinate * ratioNew;
    // 转换中心点到新坐标系
    vec2 centerNew = center * ratioNew;
    // 转换宽高到新坐标系
    vec2 sizeNew = size * ratioNew;

    // 坐标范围转换到[-1, 1]
    textureCoordinateNew = textureCoordinate*2.0-1.0;
    centerNew = center*2.0-1.0;

    // 根据宽高比缩放坐标，使刻度一致
    textureCoordinateNew = textureCoordinateNew * ratioNew;
    centerNew = centerNew * ratioNew;


    // 旋转
    float t = PI*2.0*0.15;// 先将五角星转正
    t = t + rotate;
    mat2 rotateMat = mat2(cos(t), -sin(t), sin(t), cos(t));
    textureCoordinateNew = rotateMat * textureCoordinateNew;
    centerNew = rotateMat * centerNew;

    // star shape
    float d = polarStar( textureCoordinateNew , centerNew, length(sizeNew));

    // 测试圆心位置
/*    float org = distance(textureCoordinateNew, centerNew);
    if(org<0.05){
        d = edge*2.0+0.05;
    }*/

    return smoothstep(-edge, edge, d);
}

vec2 pointList[20];
int pointCount;
// 检测point是否在pointList组成的多边形内
// 返回值：0.0=在多边形内；1.0=在多边形外
float pointInPolygon(vec2 point) {
    float inIt = 1.0;

    int i;
    int j = pointCount-1;
    float oddNodes = 0.0;
    for (i=0; i<pointCount; i++) {
        if(pointList[i].y<point.y && pointList[j].y>=point.y
        || pointList[j].y<point.y && pointList[i].y>=point.y) {
            if(pointList[i].x+(point.y-pointList[i].y)/(pointList[j].y-pointList[i].y)*(pointList[j].x-pointList[i].x)<point.x) {
                oddNodes = oddNodes+1.0;
            }
        }
        j=i;
    }

    if (mod(oddNodes, 2.0) >= 0.5){
        inIt = 0.0;
    }
    return inIt;
}
vec2 XyOfDegree( float ang ){
    return vec2( cos(ang), sin(ang) );
}
//五边形
float DoPentagon(){
    float inIt = 0.0;

    float radius = length(size);
    vec2 ratioNew = ratio<1.0?vec2(1.0, 1.0 / ratio):vec2(ratio, 1.0);
    vec2 centerNew = center*2.0-1.0;
    centerNew = centerNew * ratioNew;
    vec2 textureCoordinateNew = textureCoordinate*2.0-1.0;
    textureCoordinateNew = textureCoordinateNew * ratioNew;

    pointCount = 5;
    pointList[0] = XyOfDegree(-PI/2.0 + rotate) * radius + centerNew;
    pointList[1] = XyOfDegree(PI*2.0/5.0-PI/2.0 + rotate) * radius + centerNew;
    pointList[2] = XyOfDegree(PI*2.0/5.0*2.0-PI/2.0 + rotate) * radius + centerNew;
    pointList[3] = XyOfDegree(PI*2.0/5.0*3.0-PI/2.0 + rotate) * radius + centerNew;
    pointList[4] = XyOfDegree(PI*2.0/5.0*4.0-PI/2.0 + rotate) * radius + centerNew;

    inIt = pointInPolygon(textureCoordinateNew);

    return inIt;
}

// 心形
float DoHeart(){
    float inIt = 0.0;

    vec2 ratioNew = ratio<1.0?vec2(1.0, 1.0 / ratio):vec2(ratio, 1.0);

    vec2 centerNew = vec2(center.x, 1.0-center.y);
    centerNew = centerNew * ratioNew;
    vec2 textureCoordinateNew = vec2(textureCoordinate.x, 1.0-textureCoordinate.y);
    textureCoordinateNew = textureCoordinateNew * ratioNew;
    vec2 sizeNew = size * ratioNew;

    mat2 rotateMat = mat2(cos(rotate), -sin(rotate), sin(rotate), cos(rotate));
    textureCoordinateNew = rotateMat * textureCoordinateNew;
    centerNew = rotateMat * centerNew;

    // 标准化像素坐标
    vec2 q = textureCoordinateNew - centerNew;

    // 控制心形宽高比
    float heightHeart = 0.9;

    // 转换当前点位置
    q.y = ((1.0/heightHeart)*q.y - abs(q.x) * sqrt(50.0- abs(q.x))/10.0);

    // 心形半径
    float r = min(sizeNew.x, sizeNew.y)/2.0;

    //
    inIt = smoothstep(r, r+edge,length( q ));

    return inIt;
}


// pos: 当前点
// org: 中心点
// size: 宽高
// radius: 夹角弧度
float roundedRectangle(vec2 pos, vec2 org, vec2 size, float radius){
    vec2 hSize = size - radius;
    float d = length(max(abs(pos - org),hSize)-hSize);
    return d;
}
// https://www.shadertoy.com/view/7lBXWm
float DoRoundedRectangle(float radius){
    float inIt = 0.0;
    // 标准化像素坐标
    vec2 ratioNew = ratio<1.0?vec2(1.0, 1.0 / ratio):vec2(ratio, 1.0);
    vec2 centerNew = center*2.0-1.0;
    centerNew = centerNew * ratioNew;
    vec2 textureCoordinateNew = textureCoordinate*2.0-1.0;
    textureCoordinateNew = textureCoordinateNew * ratioNew;
    vec2 sizeNew = size*ratioNew;

    mat2 rotateMat = mat2(cos(rotate), -sin(rotate), sin(rotate), cos(rotate));
    textureCoordinateNew = rotateMat * textureCoordinateNew;
    centerNew = rotateMat * centerNew;

    float d = roundedRectangle(textureCoordinateNew, centerNew, sizeNew, radius);

    inIt = smoothstep(max(0.0, radius-edge/2.0), radius+edge/2.0, d);
    return inIt;
}


float singleTest(vec2 point, vec2 center, float size){
    const float two_pi = PI * 2.;
    const float leaves = 7.;

    vec2 offset = point - center;
    float dist = length(offset);
    float prop_y = dist/size;
/*    if (dist < size-edge){
        return 0.0;
    }
    if (prop_y > 1.0+edge){
        return 1.0;
    }*/

    float angle = atan(offset.x, offset.y);
    float prop_x = mod(((angle / two_pi) * leaves) + 0.5, 1.);
    float from_mid = abs(prop_x - 0.5) * 2.;
    float from_apex = from_mid - 0.2;
    float threshold = max(1. - 4. * from_mid,
    1. - 0.6*(from_apex * from_apex)
    );

    return smoothstep(threshold, threshold+edge, prop_y);
}

float DoFlowerShape(){
    float inIt = 0.0;

    // 标准化像素坐标
    vec2 ratioNew = ratio<1.0?vec2(1.0, 1.0 / ratio):vec2(ratio, 1.0);
    vec2 centerNew = center * ratioNew;
    vec2 textureCoordinateNew = textureCoordinate * ratioNew;
    vec2 sizeNew = size*ratioNew;

    mat2 rotateMat = mat2(cos(rotate), -sin(rotate), sin(rotate), cos(rotate));
    textureCoordinateNew = rotateMat * textureCoordinateNew;
    centerNew = rotateMat * centerNew;

    float brightness = 0.;
    float radius = min(sizeNew.x, sizeNew.y)/2.0;
    inIt = singleTest(textureCoordinateNew, centerNew, radius);

    return inIt;
}
void main (void) {

    vec4 rgba = texture2D(inputImageTexture, textureCoordinate);
    vec4 noColor = vec4(1.0, 1.0, 1.0, 0.0);
    float noColorWeight = 0.0;
    if(shape == 1){
        noColorWeight = doCircular();
    } else if(shape == 2){
        noColorWeight = DoHeart();
    } else if(shape == 3){
        noColorWeight = DoPentagon();
    } else if(shape == 4){
        noColorWeight = doFivePointedStar();
    } else if(shape == 5){
        noColorWeight = DoRoundedRectangle(0.0);
    } else if(shape == 6){
        noColorWeight = DoRoundedRectangle(0.25);
    } else if(shape == 7){
        noColorWeight = DoFlowerShape();
    }





/*
    // 测试用，看看画的范围对不对
    if ((abs(textureCoordinate.x-center.x)>size.x/2.0 && abs(textureCoordinate.x-center.x)<size.x/2.0+0.01)
    || (abs(textureCoordinate.y-center.y)>size.y/2.0 && abs(textureCoordinate.y-center.y)<size.y/2.0+0.01)
    ){
        noColor = vec4(1.0, 0.0, 0.0, 1.0);
        noColorWeight = 1.0;
    } else if(abs(textureCoordinate.x-center.x)>size.x/2.0+0.02 || abs(textureCoordinate.y-center.y)>size.y/2.0+0.02){
        // noColorWeight = 0.0;
    }
*/

    gl_FragColor = mix(rgba, noColor, noColorWeight);
}