precision highp float;
const float M_PI = 3.14159;

varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

//要画多少条线
uniform float linenum;

//将x*linenum归一化处理，判断宽度 x是线的宽度
uniform float linewidth;

//y轴对称
uniform int iscenter;
//绘制下落的高点
uniform int drawdown;
// 画方块,y部分是间隔(0-0.5)，x部分是分几段
uniform vec2 userect;
float usealpha = 1.0;
uniform int type;
//镜像
//0 不镜像
//1 0-->1 1-->0
//2 1-->0 0-->1
uniform int mirror;
//没传入数据
uniform int empty;
uniform vec2 circle;
uniform float ratio;

uniform vec4 colors0;
uniform vec4 colors1;
uniform vec4 colors2;
uniform vec4 colors3;
uniform vec4 colors4;
uniform vec4 colors5;
uniform vec4 colorsinfo;
uniform float progress;
uniform vec2 onestep;

bool usepoint;

vec4 getcolor(float pos) {
    if (pos < 1.0) {
        return colors0;
    }
    if (pos < 2.0) {
        return colors1;
    }
    if (pos < 3.0) {
        return colors2;
    }
    if (pos < 4.0) {
        return colors3;
    }
    if (pos < 5.0) {
        return colors4;
    }
    if (pos < 6.0) {
        return colors5;
    }
    return vec4(0.0);
}
vec4 setcolor(float x, float y) {
    vec4 rtn;
    //渐变颜色
    vec4 color1;
    float colorsize = colorsinfo.x;
    if (colorsize == 1.0) {
        rtn = colors0 * usealpha;
        return rtn;
    }

    vec4 color2;
    float ishorizontal = colorsinfo.z;
    bool istimechange = colorsinfo.a > 0.0;
    float usevalue = (ishorizontal == 0.0 || ishorizontal == 2.0) ? x : y;
    if (ishorizontal == 0.0 || ishorizontal == 3.0) {
        usevalue = 1.0 - usevalue;
    }

    //不渐变，color1与color2一样
    usepoint = colorsinfo.y > 0.0;
    if (istimechange) {
        if (!usepoint) {
            bool change = !(type == 8 && (ishorizontal == 0.0 || ishorizontal == 2.0));
            if (change) {
                usevalue *= colorsize / (colorsize + 1.0);
            }
        }
        usevalue += progress;
        usevalue = fract(usevalue);
    }


    if (usepoint) {
        float colorpos = usevalue * colorsize;
        float color1pos = floor(colorpos);
        color1 = getcolor(color1pos);
        rtn = color1;
    } else {
        float colorpos;
        float color1pos;
        float color2pos;
        if (istimechange) {
            colorpos = usevalue * colorsize;
            color1pos = floor(colorpos);
            color2pos = color1pos + 1.0;
            if (color2pos > colorsize - 1.0) {
                color2pos = 0.0;
            }
        } else {
            colorsize -= 1.0;
            colorpos = usevalue * colorsize;
            color1pos = floor(colorpos);
            color2pos = color1pos + 1.0;
        }
        color1 = getcolor(color1pos);
        color2 = getcolor(color2pos);
        rtn = mix(color1, color2, fract(colorpos));
    }
    return rtn * usealpha;
}


float getmirrorx(float x) {
    if (mirror == 1) {
        if (x > 0.5) {
            x = 1.0 - x;
        }
        //        x *= 2.0;
    } else if (mirror == 2) {
        if (x < 0.5) {
            x = 0.5 - x;
        } else {
            x -= 0.5;
        }
        //        x *= 2.0;
    }
    return x;
}


float getvalue(sampler2D Texture, float xpos, bool usemin) {
    if (empty > 0) {
        return 0.01;
    }
    float v = texture2D(Texture, vec2(xpos, 0.5)).r;
    float addmin = 0.0;
    if (usemin) {
        addmin = 0.01;
        if (type == 3) {
            addmin = 0.01;
        } else if (type == 8) {
            addmin = 0.02;
        }
    }

    return max(v, addmin);
    //    return 1.0;
}
float getvalue(sampler2D Texture, float xpos) {
    return getvalue(Texture, xpos, true);
}

// 计算二维向量与原点之间的角度（弧度）
float angleWithOrigin(vec2 pos) {
    float x = pos.x;
    float y = pos.y;
    if (x == 0.0 && y == 0.0) {
        return 0.0; // 原点的情况，角度为0
    }

    float angle_rad;

    if (x == 0.0) {
        if (y > 0.0) {
            angle_rad = M_PI / 2.0; // y 轴正方向
        } else {
            angle_rad = -M_PI / 2.0; // y 轴负方向
        }
    } else {
        angle_rad = atan(y / x); // 其他情况
    }

    if (x < 0.0) {
        angle_rad += M_PI; // 调整角度
    }

    return angle_rad;
}




void main() {
    vec2 pos = textureCoordinate / 1.0;
    //    if(true){
    //        float value=getvalue(inputImageTexture,textureCoordinate.x);
    //        gl_FragColor = vec4(value);
    //        return;
    //    }
    //    波浪
    if (type == 3) {
        float valuex = getmirrorx(pos.x);
        float value = getvalue(inputImageTexture, valuex);

        if (value > pos.y) {
            float cha = value - pos.y;
            if (cha > 0.02) {
                usealpha = 0.5;
            }
            //            if (cha < 0.01) {
            //                usealpha = cha * 100.0;
            //            }
            gl_FragColor = setcolor(pos.x, pos.y);
        } else {
            //            gl_FragColor = vec4(value);
            gl_FragColor = vec4(0.0);
        }
        return;
    }
    //    波浪x镜像
    if (type == 10) {
        float valuex = getmirrorx(pos.x);
        float value = getvalue(inputImageTexture, valuex);
        float valuey;
        if (pos.y > 0.5) {
            valuey = (pos.y - 0.5) * 2.0;
        } else {
            valuey = (0.5 - pos.y) * 2.0;
        }
        if (value > valuey) {
            float cha = value - valuey;
            if (cha < 0.01) {
                usealpha = cha * 100.0;
            }
            //            usealpha *= (1.0 - textureCoordinate.x);
            gl_FragColor = setcolor(pos.x, valuey);
        } else {
            gl_FragColor = vec4(0.0);
        }
        return;
    }
    //    波浪y镜像
    if (type == 9) {

        float value = getvalue(inputImageTexture, pos.x);
        pos.x = 1.0 - pos.x;
        float valuemirror = getvalue(inputImageTexture, pos.x);
        float maxvalue = max(value, valuemirror);
        if (maxvalue > pos.y) {
            if (value > pos.y) {
                usealpha = 1.0;
            } else if (valuemirror > pos.y) {
                usealpha = 0.5;
            }
            gl_FragColor = setcolor(pos.x, pos.y);
        } else {
            gl_FragColor = vec4(0.0);
        }
        return;
    }
    //    7曲线 11折线
    if (type == 7 || type == 11) {
        vec2 pos2 = vec2(pos);
        pos2.x = getmirrorx(pos.x);

        float linew = type == 11 ? 0.015 : 0.02;
        vec2 ratiov = vec2(0.001);
        if (ratio > 1.0) {
            ratiov.x /= ratio;
        } else {
            ratiov.y *= ratio;
        }
        float value = getvalue(inputImageTexture, pos2.x);
        if (abs(pos2.y - value) < 0.2) {
            float startv = -4.0;
            float endv = -startv;
            for (float i = startv; i <= endv; i++) {
                for (float j = startv; j <= endv; j++) {
                    vec2 add = vec2(i, j) * ratiov;
                    vec2 usepos = add + pos2;
                    value = getvalue(inputImageTexture, usepos.x);
                    float cha = abs(pos.y - value);
                    if (cha < linew) {
                        gl_FragColor = setcolor(pos.x, pos.y);
                        return;
                    }
                }
            }
        }
        gl_FragColor = vec4(0.0);
        return;
    }

    //    圆形
    if (type == 8) {
        vec2 center = vec2(0.5);
        if (ratio > 1.0) {
            pos.x = (pos.x - 0.5) * ratio + 0.5;
        } else {
            pos.y = (pos.y - 0.5) / ratio + 0.5;
        }
        vec2 cha = pos - center;
        float tocenter = length(cha);
        if (tocenter < 0.25 || tocenter > 0.5) {
            gl_FragColor = vec4(0.0);
        } else {
            tocenter -= 0.25;
            tocenter *= 4.0;
            float useposy = tocenter;
            // 计算向量与原点之间的角度（弧度）
            float angle_rad = angleWithOrigin(cha);
            // 归一化 0-1
            float useposx = (angle_rad + M_PI / 2.0) / (M_PI) / 2.0;
            useposx = getmirrorx(useposx);
            float value = getvalue(inputImageTexture, useposx);
            if (useposy <= value) {
                gl_FragColor = setcolor(useposx, useposy);
            } else {
                gl_FragColor = vec4(0.0);
            }
        }
        return;
    }
    pos.x = getmirrorx(pos.x);

    float outx = pos.x * linenum;
    //    float x = fract(outx);
    float x = outx - floor(outx);
    //    距离中心的距离
    float xpos = abs(0.5 - x);
    //    距离中心的距离是不是小于linewidth
    float chax = (linewidth / 2.0) - xpos;

    //    线柱结合
    if (type == 15) {
        vec2 ratiov = vec2(0.001);
        float linew = 0.01;
        if (ratio < 1.0) {
            ratiov.x /= ratio;
        } else {
            ratiov.y *= ratio;
        }
        for (float i = -2.0; i < 2.5; i++) {
            for (float j = -2.0; j < 2.5; j++) {
                vec2 add = vec2(i, j) * ratiov;
                vec2 usepos = add + pos;
                float value = getvalue(inputImageTexture, usepos.x);
                float cha = abs(pos.y - value);
                if (cha < linew) {
                    gl_FragColor = setcolor(pos.x, pos.y);
                    return;
                }
            }
        }
    }

    if (chax > 0.0) {
        //        float falloff = 0.1;
        //        if (chax < falloff && userect.x < 0.0) {
        //            usealpha = chax * 10.0;
        //        }
        float toborderx;
        float tobordery;
        bool isiny;
        outx = outx - x;
        float usex = outx / linenum;
        pos.x = usex;
        pos.y = 0.5;
        float value = getvalue(inputImageTexture, usex);
        //        柱状原点
        if (type == 12) {
            pos.y = textureCoordinate.y;
            float radios = (linewidth / 2.0);
            //  x半径
            float radiosx = 1.0 / linenum * radios;
            //            y半径
            float radiosy = radiosx * ratio;
            float topline = max(radiosy * 2.0, value);
            //            高过了最高，直接返回
            if (pos.y > topline) {
                gl_FragColor = vec4(0.0);
                return;
            }

            float circlex = xpos / radios;
            //            在最底下的部分一直画圆
            if (pos.y <= radiosy) {
                float circley = (radiosy - pos.y) / radiosy;
                if (length(vec2(circlex, circley)) <= 1.0) {
                    gl_FragColor = setcolor(pos.x, pos.y);
                    //                    gl_FragColor=vec4(0.0,0.0,0.0,1.0);
                } else {
                    gl_FragColor = vec4(0.0);
                }
                return;
            }
            //            上半部分的圆
            topline -= radiosy;
            if (pos.y >= topline) {
                float circley = (topline - pos.y) / radiosy;
                if (length(vec2(circlex, circley)) <= 1.0) {
                    gl_FragColor = setcolor(pos.x, pos.y);
                    //                    gl_FragColor=vec4(0.0,0.0,0.0,1.0);
                } else {
                    gl_FragColor = vec4(0.0);
                }
            } else {
                gl_FragColor = setcolor(pos.x, pos.y);
            }
            return;
        }
        //        柱状原点对称
        if (type == 18 || type == 17) {
            value = getvalue(inputImageTexture, usex, false);
            pos.y = abs(textureCoordinate.y - 0.5);

            float radios = (linewidth / 2.0);
            //  x半径
            float radiosx = 1.0 / linenum * radios;
            float circlex = xpos / radios;

            //            y半径
            float radiosy = radiosx * ratio;
            float topline;
            value = value * (0.5 - radiosy);
            //   topline = value;
            //   value /= 2.0;
            topline = radiosy + value;
            //  高过了最高，直接返回
            if (pos.y > topline) {
                gl_FragColor = vec4(0.0);
                return;
            }
            //            上半部分的圆
            if (pos.y >= value) {
                float circley = (pos.y - value) / radiosy;
                if (length(vec2(circlex, circley)) <= 1.0) {
                    gl_FragColor = setcolor(pos.x, textureCoordinate.y);
                    //                    gl_FragColor = vec4(0.0, 1.0, 0.0, 1.0);
                } else {
                    gl_FragColor = vec4(0.0);
                }
            } else {
                gl_FragColor = setcolor(pos.x, textureCoordinate.y);
                //                gl_FragColor = vec4(1.0, 0.0, 0.0, 1.0);
            }
            return;
        }

        if (type == 13) {
            pos.y = textureCoordinate.y;
            float ypos = pos.y / userect.x;
            float valuepos = floor(value / userect.x) + 1.0;
            isiny = ypos < valuepos && (fract(ypos) < userect.y);
            if (isiny) {
                gl_FragColor = setcolor(pos.x, pos.y);
            } else {
                gl_FragColor = vec4(0.0);
            }
            return;
        }
        if (type == 14) {
            pos.y = textureCoordinate.y;
            float centerh = userect.x;
            if (abs(0.5 - pos.y) < centerh / 2.0) {
                gl_FragColor = vec4(0.0);
            } else {
                if (pos.y < 0.5) {
                    pos.y = (0.5 - pos.y) * 2.0;
                    usealpha = 0.5;
                } else {
                    pos.y = (pos.y - 0.5) * 2.0;
                }
                pos.y = (pos.y - centerh) / (1.0 - centerh);
                if (pos.y < value) {
                    gl_FragColor = setcolor(pos.x, pos.y);
                } else {
                    gl_FragColor = vec4(0.0);
                }
            }
            return;
        }
        if (type == 15) {
            value = getvalue(inputImageTexture, getmirrorx(textureCoordinate.x));
            pos.y = textureCoordinate.y;
            if (pos.y < value) {
                gl_FragColor = setcolor(pos.x, pos.y);
            } else {
                gl_FragColor = vec4(0.0);
            }
            return;
        }

        if (type == 16) {
            pos.y = textureCoordinate.y;
            float cha = abs(value - pos.y);
            if (userect.x > cha) {
                //                cha = 1.0-cha / userect.x;
                //                if (cha < 0.5) {
                //                    usealpha = cha / 0.5;
                //                }
                gl_FragColor = setcolor(pos.x, pos.y);
            } else {
                gl_FragColor = vec4(0.0);
            }
            return;
        }


        //value = 1.0;
        float mixy;
        //       userect的整数与小数部分
        float rectsize = userect.x;
        float voids = userect.y;

        if (drawdown > 0) {
            float valuedown = getvalue(inputImageTexture2, pos.x);
            if (valuedown > value && rectsize > 0.0) {
                tobordery = abs(textureCoordinate.y - valuedown);
                isiny = tobordery * rectsize < voids / 2.0;

                if (isiny) {
                    if (type == 1) {
                        tobordery /= circle.y;
                        isiny = tobordery < 1.0;
                        float posx = min(x, 1.0 - x);
                        float borderx = (1.0 - linewidth) / 2.0;
                        toborderx = (posx - borderx) / linenum;
                        toborderx /= circle.x;
                        if (isiny && toborderx < 1.0) {
                            toborderx = 1.0 - toborderx;
                            isiny = toborderx * toborderx + tobordery * tobordery < 1.0;
                        }
                    }
                    if (isiny) {
                        gl_FragColor = setcolor(pos.x, valuedown);
                        // gl_FragColor = vec4(0.0);
                        // gl_FragColor.a = 1.0;
                        return;
                    }
                    gl_FragColor = vec4(0.0);
                    return;
                }
            }
        }

        float colory = textureCoordinate.y;
        if (rectsize > 0.0) {
            colory = floor(textureCoordinate.y * rectsize) / rectsize;
        }
        if (iscenter > 0) {
            mixy = abs(0.5 - textureCoordinate.y) * 2.0;
            isiny = mixy < value;
            colory = mixy;
        } else {
            isiny = colory <= value;
        }

        if (isiny) {
            if (rectsize > 0.0) {
                float usey = fract(textureCoordinate.y * rectsize);
                voids /= 2.0;
                isiny = abs(voids - usey) <= voids;
                //            绘制圆角
                if (isiny && type == 1) {
                    tobordery = abs(voids - usey) / rectsize;
                    float posx = min(x, 1.0 - x);
                    float borderx = (1.0 - linewidth) / 2.0;
                    toborderx = (posx - borderx) / linenum;
                    toborderx /= circle.x;
                    tobordery /= circle.y;
                    isiny = tobordery < 1.0;
                    if (isiny && toborderx < 1.0) {
                        toborderx = 1.0 - toborderx;
                        isiny = toborderx * toborderx + tobordery * tobordery < 1.0;
                    }
                }
            }
        }
        if (isiny) {
            gl_FragColor = setcolor(pos.x, colory);
            return;
        }
    }

    //    else{
    //        gl_FragColor = vec4(1.0);
    //        return;
    //    }
    gl_FragColor = vec4(0.0);

}
