precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float alpha;
uniform float ratio;

uniform vec2 center;
uniform vec2 wh;
uniform float roate;
//混合模式
uniform int type;
uniform int isfromanim;

vec2 center2 = vec2(0.5);

//  叠加 3 叠加与强光运算相同，就是两个颜色位置换了一下
vec4 getOverlay(vec4 baseColor, vec4 blendColor) {
    vec4 lumCoeff = vec4(0.2125, 0.7154, 0.0721, 1.0);
    float luminance = dot(baseColor.rgb, lumCoeff.rgb);
    vec4 rtn;
    if (luminance < 0.45)
    {
        rtn = 2.0 * baseColor * blendColor;
    }
    else if (luminance > 0.55)
    {
        rtn = vec4(1.0) - 2.0 * ((vec4(1.0) - baseColor) * (vec4(1.0) - blendColor));
    }
    else
    {
        vec4 colorT1 = 2.0 * baseColor * blendColor;
        vec4 colorT2 = vec4(1.0) - 2.0 * ((vec4(1.0) - baseColor) * (vec4(1.0) - blendColor));
        rtn = mix(baseColor, blendColor, (luminance - 0.45) * 10.0);
    }
    return rtn;
}

//  强光 6 叠加与强光运算相同，就是两个颜色位置换了一下
vec4 getHARDLIGHT(vec4 baseColor, vec4 blendColor) {
    vec4 lumCoeff = vec4(0.2125, 0.7154, 0.0721, 1.0);
    float luminance = dot(blendColor.rgb, lumCoeff.rgb);
    vec4 rtn;
    if (luminance < 0.45) {
        rtn = 2.0 * baseColor * blendColor;
    }
    else if (luminance > 0.55) {
        rtn = vec4(1.0) - 2.0 * ((vec4(1.0) - baseColor) * (vec4(1.0) - blendColor));
    }
    else {
        vec4 colorT1 = 2.0 * baseColor * blendColor;
        vec4 colorT2 = vec4(1.0) - 2.0 * ((vec4(1.0) - baseColor) * (vec4(1.0) - blendColor));
        rtn = mix(baseColor, blendColor, (luminance - 0.45) * 10.0);
    }
    return rtn;
}

//https://www.freesion.com/article/9005449593/
//https://zhuanlan.zhihu.com/p/345014180
//  正常 0
//  滤色 1
//  变暗 2
//  叠加 3
//  正片叠底 4
//  变亮 5
//  强光 6
//  柔光 7
//  线性加深 8
//  颜色加深 9
//  颜色减淡 10
vec4 getmixcolor(vec4 baseColor, vec4 blendColor) {
    vec4 rtn = blendColor;
    if (type == 0) {
        rtn = blendColor;
    } else if (type == 1) {
        rtn = vec4(1.0) - ((vec4(1.0) - baseColor) * (vec4(1.0) - blendColor));
    } else if (type == 2) {
        rtn = min(baseColor, blendColor);
    } else if (type == 3) {
        rtn = getOverlay(baseColor, blendColor);
    } else if (type == 4) {
        rtn = baseColor * blendColor;;
    } else if (type == 5) {
        rtn = max(baseColor, blendColor);
    } else if (type == 6) {
        rtn = getHARDLIGHT(baseColor, blendColor);
    } else if (type == 7) {
        rtn = 2.0 * baseColor * blendColor + baseColor * baseColor - 2.0 * baseColor * baseColor * blendColor;
    } else if (type == 8) {
        rtn = baseColor + blendColor - vec4(1.0);
    } else if (type == 9) {
        rtn = vec4(1.0) - (vec4(1.0) - baseColor) / blendColor;
    } else if (type == 10) {
        rtn = baseColor / (vec4(1.0) - blendColor);
    }
    return rtn;
}

vec2 roatepos(vec2 pos, float r) {
    float aSin = sin(r);
    float aCos = cos(r);

    mat2 rotMat = mat2(aCos, -aSin, aSin, aCos);
    mat2 scaleMat = mat2(ratio, 0.0, 0.0, 1.0);
    mat2 scaleMatInv = mat2(1.0 / ratio, 0.0, 0.0, 1.0);

    pos = scaleMatInv * rotMat * scaleMat * pos;
    return pos;
}

vec2 getuseuv() {
    vec2 use = textureCoordinate - center;
    use.x /= wh.x;
    use.y /= wh.y;
    if (roate != 0.0) {
        use = roatepos(use, roate);
    }
    use += vec2(0.5);
    return use;
}
void main() {
    vec4 base = texture2D(inputImageTexture, textureCoordinate);
    vec2 useuv = getuseuv();
    if (abs(useuv.x - 0.5) > 0.5 || abs(useuv.y - 0.5) > 0.5) {
        gl_FragColor = base;
    } else {
        vec4 color2 = texture2D(inputImageTexture2, useuv);
        float olda= color2.a;
        if (olda == 0.0) {
            gl_FragColor = base;
            return;
        }
        if (isfromanim > 0) {
            color2.rgb /= olda;
        }
        color2.a=1.0;
        color2 = getmixcolor(base, color2);
        gl_FragColor = mix(base, color2, olda * alpha);
    }

}