// License: MIT
// Author: rectalogic
// ported by gre from https://gist.github.com/rectalogic/b86b90161503a0023231

// Converted from https://github.com/rectalogic/rendermix-basic-effects/blob/master/assets/com/rendermix/CrossZoom/CrossZoom.frag
// Which is based on https://github.com/evanw/glfx.js/blob/master/src/filters/blur/zoomblur.js
// With additional easing functions from https://github.com/rectalogic/rendermix-basic-effects/blob/master/assets/com/rendermix/Easing/Easing.glsllib

// 3-1
// https://gl-transitions.com/editor/CrossZoom

precision highp float;
varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float progress;
const float PI = 3.1415926;
uniform  vec4 move1;

vec4 getToColor(vec2 uv) {
    return texture2D(inputImageTexture2, uv);
}
vec4 getFromColor(vec2 uv) {
    return texture2D(inputImageTexture, uv);
}


uniform float ratio;//视图的真实比例
//float ratio=1.0;
//椭中心点
float centerx = 0.5;
//float centery=1.0;
float centery = 0.5;
//椭圆中心点的长径
float radius = 0.6;


/*
高中课本在平面直角坐标系中，用方程描述了椭圆，椭圆的标准方程为：x^2/a^2+y^2/b^2=1
其中a>0，b>0。
a、b中较大者为椭圆长半轴长，较短者为短半轴长
（椭圆有两条对称轴，对称轴被椭圆所截，有两条线段，它们分别叫椭圆的长半轴和短半轴）
当a>b时，焦点在x轴上，焦距为2*(a^2-b^2)^0.5，准线方程是x=a^2/c和x=-a^2/c */
bool isin(vec2 c2) {
    //    先判断在不在矩形范围内
    //    中心点 距离xy轴的距离
    float rx = radius;
    float ry = radius;
    vec2 center = vec2(centerx, centery);
    //    距离中心点的相对坐标xy
    vec2 cha = c2 - center;
    //    宽>高
    if (ratio > 1.0) {
        rx = radius / ratio;
    } else {
        ry = radius * ratio;
    }
    if (abs(cha.x) > rx || abs(cha.y) > ry) {
        return false;
    } else {
        bool isin = (cha.x * cha.x) / (rx * rx) + (cha.y * cha.y) / (ry * ry) <= 1.0;
        return isin;
    }
}
float rand(vec2 co) {
    return fract(sin(dot(co.xy, vec2(12.9898, 78.233))) * 43758.5453);
}

vec2 doroate(vec2 c2, float roate, float scale) {
    vec2 center = vec2(centerx, centery);

    //计算旋转角度
    vec2 uv2 = c2 - center;
    if (ratio > 1.0) {
        uv2.x = uv2.x * ratio;
    } else {
        uv2.y = uv2.y / ratio;
    }
    float c = cos(roate);
    float s = sin(roate);
    float dx2 = uv2.x * c + uv2.y * s;
    float dy2 = -uv2.x * s + uv2.y * c;
    if (ratio > 1.0) {
        dx2 = dx2 / ratio;
    } else {
        dy2 = dy2 * ratio;
    }
    vec2 rtn = vec2(center.x + dx2 / scale, center.y + dy2 / scale);
    return rtn;
}

void main() {
    vec2 texCoord = textureCoordinate.xy / vec2(1.0).xy;
    //    是否在最里面椭圆里面
    bool isin = isin(texCoord);

//    float usepro = progress;
//    if (!isin) {
//        if (progress >= 0.5) {
//            usepro = (1.0 - progress) * 2.0;
//            usepro = 1.0 - usepro * usepro / 2.0;
//        } else {
//            usepro = progress * 2.0;
//            usepro = usepro * usepro / 2.0;
//        }
//    }
//    usepro = getresetpro(usepro);

    float usepro=isin?move1[0]:move1[1];

    //    180度＝π弧度
    float roate = usepro * PI * 2.0;
    vec2 offuse = doroate(texCoord, roate, 1.0);
    if (offuse.x < 0.)offuse.x = -offuse.x;
    if (offuse.y < 0.)offuse.y = -offuse.y;
    if (offuse.x > 1.)offuse.x = 2.0 - offuse.x;
    if (offuse.y > 1.)offuse.y = 2.0 - offuse.y;


    vec3 color = vec3(0.0);
    float total = 0.0;
    float offset = rand(offuse);
    float num = 10.0;
    usepro = progress > 0.5 ? 1.0 - progress : progress;

    for (float t = 0.0; t <= num; t++) {
        float percent = (t + offset) / num;
        float weight = 4.0 * (percent - percent * percent);
        float needroate = roate - (num - t) * 0.05 * usepro;
        if (!isin && progress < 0.15 && needroate < 0.) {
            if (total == 0.0) {
                gl_FragColor = progress < 0.5 ?
                texture2D(inputImageTexture, texCoord) :
                texture2D(inputImageTexture2, texCoord);
            } else {
                gl_FragColor = vec4(color / total, 1.0);
            }
            return;
        }
        vec2 movepos = doroate(texCoord, needroate, 1.0);
        if (movepos.x < 0.)movepos.x = -movepos.x;
        if (movepos.y < 0.)movepos.y = -movepos.y;
        if (movepos.x > 1.)movepos.x = 2.0 - movepos.x;
        if (movepos.y > 1.)movepos.y = 2.0 - movepos.y;
        //        movepos =texCoord;
        color += (progress < 0.5 ?
        texture2D(inputImageTexture, movepos).rgb :
        texture2D(inputImageTexture2, movepos).rgb) * weight;
        total += weight;
    }
    gl_FragColor = vec4(color / total, 1.0);


}



