//矩形转场 左边4个旋转 右边对称并反转
precision highp float;
varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform lowp float progress;
uniform lowp float ratio;

vec4 getToColor(vec2 uv) {
    return texture2D(inputImageTexture2, uv);
}
vec4 getFromColor(vec2 uv) {
    return texture2D(inputImageTexture, uv);
}

vec2 rotateUV(vec2 uv, float degree, vec2 rotateCenter, float ratio) {

    float arc = degree / 180.0 * 3.14159;

    float aSin = sin(arc);
    float aCos = cos(arc);

    mat2 rotMat = mat2(aCos, -aSin, aSin, aCos);
    mat2 scaleMat = mat2(ratio, 0.0, 0.0, 1.0);
    mat2 scaleMatInv = mat2(1.0 / ratio, 0.0, 0.0, 1.0);

    vec2 pos = uv;

    pos -= rotateCenter;
    pos = scaleMatInv * rotMat * scaleMat * pos;
    pos += rotateCenter;
    return pos;
}

//矩形
float sdfBox(vec2 uv, vec2 size, vec2 center)
{
    vec2 pos = uv - center;
    vec2 d = abs(pos) - size;
    return length(max(d, 0.0)) + min(max(d.x, d.y), 0.0);
}



//带旋转的矩形
//uv            坐标
//size          矩形的尺寸 即矩形长/宽的一半
//center        矩形的中心点
//rotateCenter  旋转中心点，相对center， (0.0， 0.0)即为矩形中心点旋转
//degree        矩形旋转的角度
//ratio         画布的比例
float sdfRotateBox(vec2 uv, vec2 size, vec2 center, vec2 rotateCenter, float degree, float ratio)
{
    vec2 pos = uv - center;
    pos = rotateUV(pos, degree, rotateCenter, ratio);
    vec2 d = abs(pos) - size;
    return length(max(d, 0.0)) + min(max(d.x, d.y), 0.0);
}

float easeInOutCubic(float x) {
    return x < 0.5 ? (4.0 * x * x * x) : 1.0 - pow(-2.0 * x + 2.0, 3.0) / 2.0;
}
//自定义Smoothstep
float my_smoothstep(float edge0, float edge1, float x) {
    float t = clamp((x - edge0) / (edge1 - edge0), 0.0, 1.0);
    return easeInOutCubic(t);
}

void main() {

    //左右对称 并且反转
    vec2 originUV = textureCoordinate;
    vec2 uv = textureCoordinate;
    if(uv.x > 0.5) {
        uv.x = 1.0 - uv.x;
        uv.y = 1.0 - uv.y;
    }

    //矩形的个数
    float num = 4.0;
    //每个矩形动画的时间
    float duration = 0.5;
    //矩形动画之间的间隔
    float interval = (1.0 - duration) / (num - 1.0);

    //矩形的长的一半
    float sizeW = 0.5 / num / 2.0;
    //矩形宽的一半
    float sizeH = 10.0;

    //矩形中心点
    float centerH = -0.1;

    //多个矩形的拼接，用min来拼接多个sdf图形
    float minD = 0.0;

    for (float i = 0.0; i < num; i++) {
        //每一个矩形的中心点h
        float centerW = 0.5 - sizeW * (2.0 * i + 1.0);
        //每一个矩形动画的开始时间
        float startTime = interval * i;
        //每一个矩形动画的结束时间
        float endTime = startTime + duration;
        //这里是每一个矩形动画的进度，用smoothstep做的曲线，也可以仿照smoothstep自定义自己的曲线
        float percent = my_smoothstep(startTime, endTime, progress);

        //根据进度计算出当前的角度
        //左边的 90 -> 0
        float degree = 90.0 - 90.0 * percent;

        //绘制矩形
        float d = sdfRotateBox(uv, vec2(sizeW, sizeH), vec2(centerW, centerH), vec2(0.0), degree, ratio);
        //取最小值 即为多个sdf图形拼接
        minD = min(minD, d);
    }

    vec4 texColor1 = getFromColor(originUV);
    vec4 texColor2 = getToColor(originUV);
    gl_FragColor = mix(texColor2, texColor1, step(0.0, minD));
}
