//矩形转场 斜45度
precision highp float;
varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform highp float progress;
uniform highp float ratio;

vec4 getToColor(vec2 uv) {
    return texture2D(inputImageTexture2, uv);
}
vec4 getFromColor(vec2 uv) {
    return texture2D(inputImageTexture, uv);
}


vec2 rotateUV(vec2 uv, float arc, vec2 rotateCenter, float ratio) {


    float aSin = sin(arc);
    float aCos = cos(arc);

    mat2 rotMat = mat2(aCos, -aSin, aSin, aCos);
    mat2 scaleMat = mat2(ratio, 0.0, 0.0, 1.0);
    mat2 scaleMatInv = mat2(1.0 / ratio, 0.0, 0.0, 1.0);

    vec2 pos = uv;

    pos -= rotateCenter;
    pos = scaleMatInv * rotMat * scaleMat * pos;
    pos += rotateCenter;
    return pos;
}

//矩形
float sdfBox(vec2 uv, vec2 size, vec2 center)
{
    vec2 pos = uv - center;
    vec2 d = abs(pos) - size;
    return length(max(d, 0.0)) + min(max(d.x, d.y), 0.0);
}



//带旋转的矩形
//uv            坐标
//size          矩形的尺寸 即矩形长/宽的一半
//center        矩形的中心点
//rotateCenter  旋转中心点，相对center， (0.0， 0.0)即为矩形中心点旋转
//arc           弧度
//ratio         画布的比例
float sdfRotateBox(vec2 uv, vec2 size, vec2 center, vec2 rotateCenter, float arc, float ratio)
{
    vec2 pos = uv - center;
    pos = rotateUV(pos, arc, rotateCenter, ratio);
    vec2 d = abs(pos) - size;
    return length(max(d, 0.0)) + min(max(d.x, d.y), 0.0);
}

float easeInOutCubic(float x) {
    return x < 0.5 ? (4.0 * x * x * x) : 1.0 - pow(-2.0 * x + 2.0, 3.0) / 2.0;
}
//自定义Smoothstep
float my_smoothstep(float edge0, float edge1, float x) {
    float t = clamp((x - edge0) / (edge1 - edge0), 0.0, 1.0);
    return easeInOutCubic(t);
}

//这个动画用弧度来算比较简单
void main() {

    //斜向对称 180度对称
    vec2 originUV = textureCoordinate;
    vec2 uv = textureCoordinate;
    if(uv.y < uv.x) {
        uv = 1.0 - uv;
    }

    float num = 7.0;

    float duration = 0.4;

    float interval = (1.0 - duration) / (num - 1.0);


    //起始角度 即矩形卡在对角线上
    float arc = atan(ratio, 1.0) + 0.001;

    float sizeW = sin(arc) / ratio / num / 2.0;
    //非常长 long long long
    float sizeH = 1000.0;

    float centerW = -0.2;

    float minD = 0.0;

    for (float i = 0.0; i < num; i++) {
        //每一个centerW H 都往左下偏
        //不能往左偏 会造成图片中心偏左 旋转效果不明显了
//        float centerW = -0.2 - sin(-arc) * sizeW * ratio * (i * 2.0 + 1.0);
//        float centerH = -0.2 + sin(-arc) * sizeW / ratio * (i * 2.0 + 1.0);

        float dy = sizeW / sin(arc) * ratio;
        float centerH = -0.2 + (i * 2.0 + 1.0) * dy;

        float startTime = interval * i;
        float endTime = startTime + duration;
        float percent = my_smoothstep(startTime, endTime, progress);

        float currentArc = -arc * percent;
        float d = sdfRotateBox(uv, vec2(sizeW, sizeH), vec2(centerW, centerH), vec2(0.0, 0.0), currentArc, ratio);
        minD = min(minD, d);
    }

    vec4 texColor1 = getFromColor(originUV);
    vec4 texColor2 = getToColor(originUV);
    gl_FragColor = mix(texColor2, texColor1, step(0.0, minD));
}
