precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float progress;
uniform float ratio;

// 简单的伪随机函数
float random(float x) {
    return fract(sin(x) * 43758.5453123);
}

// 线性插值函数
float lerp(float a, float b, float t) {
    return a + t * (b - a);
}

// 简化的噪声函数
float noise(float x) {
    float i = floor(x);
    float f = fract(x);
    float u = f * f * (3.0 - 2.0 * f);
    return lerp(random(i), random(i + 1.0), u);
}

vec2 doroate(vec2 uv2, float roate) {
    //计算旋转角度
    if (ratio > 1.0) {
        uv2.x = uv2.x * ratio;
    } else {
        uv2.y = uv2.y / ratio;
    }
    float c = cos(roate);
    float s = sin(roate);
    float dx2 = uv2.x * c + uv2.y * s;
    float dy2 = -uv2.x * s + uv2.y * c;
    if (ratio > 1.0) {
        dx2 = dx2 / ratio;
    } else {
        dy2 = dy2 * ratio;
    }
    return vec2(dx2, dy2);
}


vec2 center = vec2(0.5);
void main() {
    float value1 = noise(progress / 100.0) - 0.5;
    float value2 = noise(progress / 100.0 + 1000.0) - 0.5;
    float value3 = noise(progress / 50.0 + 2000.0) - 0.5;

    if (ratio > 1.0) {
        value1 /= ratio;
    } else {
        value2 *= ratio;
    }


    vec2 uv = textureCoordinate.xy / 1.0 - center;
    //    uv += params.gb;
    //    uv = doroate(uv, params.r);
    uv += vec2(value1 * 0.01, value2 * 0.01);
    uv = doroate(uv, value3 * 0.015);
    uv += center;
    gl_FragColor = texture2D(inputImageTexture, uv);


}



