precision highp float;
varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform lowp float progress;
uniform float ratio;

#define GOLDEN_ANGLE 2.3999632
#define ITERATIONS 50
const mat2 rot = mat2(cos(GOLDEN_ANGLE), sin(GOLDEN_ANGLE), -sin(GOLDEN_ANGLE), cos(GOLDEN_ANGLE));


vec3 Bokeh(sampler2D tex, vec2 uv, float radius)
{
    vec3 acc = vec3(0);
    vec3 div = acc;
    float r = 1.;
    vec2 vangle = vec2(0.0, radius * .01 / sqrt(float(ITERATIONS)));
    vec2 offratio = vec2(1.0);
    if (ratio > 1.0) {
        offratio.x = 1.0 / ratio;
    } else {
        offratio.y = ratio;
    }

    for (int j = 0; j < ITERATIONS; j++)
    {
        // the approx increase in the scale of sqrt(0, 1, 2, 3...)
        r += 1. / r;
        vangle = rot * vangle;
        vec3 col = texture2D(tex, uv + (r - 1.) * vangle * offratio).xyz; /// ... Sample the image
        vec3 nowcol = col * col * 1.5; // ... Contrast it for better highlights - leave this out elsewhere.
        col = mix(col, nowcol, radius);
        vec3 bokeh = pow(col, vec3(4.0));
        acc += col * bokeh;
        div += bokeh;
    }
    return acc / div;
}

void main() {
    vec2 uv = textureCoordinate.xy / vec2(1.0);
    //    float pro = getspeedpro(progress);
    if (progress <= 0.4) {
        gl_FragColor = vec4(Bokeh(inputImageTexture, uv, progress / 0.6), 1.0);
    } else if (progress >= 0.6) {
        gl_FragColor = vec4(Bokeh(inputImageTexture2, uv, (1.0 - progress) / 0.6), 1.0);
    } else {
        vec4 color1 = vec4(Bokeh(inputImageTexture, uv, progress / 0.6), 1.0);
        vec4 color2 = vec4(Bokeh(inputImageTexture2, uv, (1.0 - progress) / 0.6), 1.0);
        gl_FragColor = mix(color1, color2, (progress - 0.4) / 0.2);
    }
}
