precision mediump float;
varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform lowp float progress;
uniform float ratio;
vec4 showcolor = vec4(1.0);

vec4 getToColor(vec2 uv) {
    return texture2D(inputImageTexture2, uv);
}
vec4 getFromColor(vec2 uv) {
    return texture2D(inputImageTexture, uv);
}

vec3 rgbToHsl(vec3 c) {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hslToRgb(vec3 c) {
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

/**
//色调分离
vec4 saturate2(vec4 color) {
    //计算灰度值
    float grayValue = color.r * 0.3 + color.g * 0.59 + color.b * 0.11;
    //转换到hsl颜色空间
    vec3 hslColor = vec3(rgbToHsl(color.rgb));
    //根据灰度值区分阴影和高光，分别处理
    if (grayValue < 0.3) {
        //添加蓝色
        if (hslColor.x < 0.68 || hslColor.x > 0.66) {
            hslColor.x = 0.67;
        }
        //增加饱和度
        hslColor.y += 0.3;
    } else if (grayValue > 0.7) {
        //添加黄色
        if (hslColor.x < 0.18 || hslColor.x > 0.16) {
            hslColor.x = 0.17;
        }
        //降低饱和度
        hslColor.y -= 0.3;
    }
    color = vec4(hslToRgb(hslColor), color.a);
    return color;
}
*/


//转换成hsl，
//h转化成0-6;向整数位偏移
//l逐渐变亮
vec4 saturate(vec4 color) {
    //转换到hsl颜色空间
    vec3 hslColor = rgbToHsl(color.rgb);
    float h=hslColor[0];
    float s=hslColor[1];
    float l=hslColor[2];

    l+=min(progress,1.0-progress);
    l=min(1.0,l);

    float bili=1.0-min(progress,1.0-progress)*2.0;

    h=h*6.0;
    float hl=floor(h);
    float cha=h-hl;
    if(cha<0.5){
        cha*=bili;
        h=hl+cha;
    }else{
        cha=(1.0-cha)*bili;
        h=hl+1.0-cha;
    }
    h/=6.0;

//    s=s*6.0;
//    float sl=floor(s);
//    cha=s-sl;
//    if(cha<0.5){
//        cha*=bili;
//        s=sl+cha;
//    }else{
//        cha=(1.0-cha)*bili;
//        s=sl+1.0-cha;
//    }
//    s/=6.0;

    if(s<0.5){
        s=s*bili;
    }else{
        s=1.0-(1.0-s)*bili;
    }


    hslColor[0]=h;
    hslColor[1]=s;
    hslColor[2]=l;
    color = vec4(hslToRgb(hslColor), color.a);
    return color;
}

//https://blog.csdn.net/ZhaDeNianQu/article/details/125239986
void main() {
    vec2 uv = textureCoordinate.xy / vec2(1.0);
    vec4 usecolor = progress < 0.5 ? getFromColor(uv) : getToColor(uv);
    vec4 usecolor2;
//    float colorLevels = 1.0;
//    usecolor2 = floor((usecolor * colorLevels) + vec4(0.5)) / colorLevels;
    gl_FragColor=saturate(usecolor);
}
