precision mediump float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float alpha;
uniform float scalex;
uniform float scaley;
uniform float roate;
uniform float tranx;
uniform float trany;
uniform float ratio;
//>0 超出部分做镜像处理<0 透明处理
uniform float mirror;
uniform vec2 rcenter;
uniform vec2 scenter;

//动态模糊的类型
uniform float motiontype;
//uniform vec2 motioncenter;
uniform vec3 motionlength;

vec2 scalepos(vec2 pos) {
    pos = (scenter - pos);
    pos.x /= scalex;
    pos.y /= scaley;
    return scenter - pos;
}

vec2 scalepos(vec2 pos, float sx, float sy) {
    pos = (scenter - pos);
    pos.x /= sx;
    pos.y /= sy;
    return scenter - pos;
}

vec2 roatepos(vec2 pos, float r) {
    float aSin = sin(r);
    float aCos = cos(r);

    mat2 rotMat = mat2(aCos, -aSin, aSin, aCos);
    mat2 scaleMat = mat2(ratio, 0.0, 0.0, 1.0);
    mat2 scaleMatInv = mat2(1.0 / ratio, 0.0, 0.0, 1.0);

    pos -= rcenter;
    pos = scaleMatInv * rotMat * scaleMat * pos;
    pos += rcenter;
    return pos;
}

//获得镜像坐标
//https://www.mk2048.com/blog/blog_ihkcc1kjjb.html
// 判断奇数偶数
// mod(x,2.0)>=1.0 false 偶数 true 奇数
//  y = mod(x,2.0) < 1.0 ? 0. : 1. ;
//  y = step(1.0, mod(x,2.0));
//  y = fract(x * .5) > 0.5
vec2 resettexCoord(vec2 rtn) {
    if (mirror <= 0.0) {
        if (rtn.x < 0.0) {
            rtn.x = 0.0;
        } else if (rtn.x > 1.0) {
            rtn.x = 1.0;
        }
        if (rtn.y < 0.0) {
            rtn.y = 0.0;
        } else if (rtn.y > 1.0) {
            rtn.y = 1.0;
        }
        return rtn;
    }
    if (mirror != 1.0) {
        if (abs(rtn.x - 0.5) > mirror - 0.5 || abs(rtn.y - 0.5) > mirror - 0.5) {
            rtn.x = -1.0;
            rtn.y = -1.0;
            return rtn;
        }
    }
    rtn.x = abs(rtn.x);
    rtn.y = abs(rtn.y);
    bool mx = mod(rtn.x, 2.0) >= 1.0;
    bool my = mod(rtn.y, 2.0) >= 1.0;
    rtn.x = mod(rtn.x, 1.0);
    rtn.y = mod(rtn.y, 1.0);
    if (mx) {
        rtn.x = 1.0 - rtn.x;
    }
    if (my) {
        rtn.y = 1.0 - rtn.y;
    }
    return rtn;
}

vec2 reset(vec2 pos) {
    pos += vec2(tranx, trany);
    if (mirror > 0.0 && mirror != 1.0) {
        if (ratio <= 1.0) {
            pos.x = (pos.x - 0.5) / ratio + 0.5;
        } else {
            pos.y = (pos.y - 0.5) * ratio + 0.5;
        }
    }

    if (scalex != 1. || scaley != 1.) {
        pos = scalepos(pos);
    }
    if (roate != 0.) {
        pos = roatepos(pos, roate);
    }

    if (mirror > 0.0) {
        pos = resettexCoord(pos);
    }
    return pos;
}



void main() {
    vec2 pos = reset(textureCoordinate);
    //    pos=textureCoordinate;
    if (abs(pos.x - 0.5) > 0.5 || abs(pos.y - 0.5) > 0.5) {
        gl_FragColor = vec4(0.0);
    } else {
        //      >0要动态模糊
        if (motiontype > 0.0) {
            float num = 10.;
            float total = 0.;
            vec4 color = vec4(0.0);
            vec2 move = vec2(0.0);
            //
            if (motiontype == 1.0) {
                move = motionlength.xy;
                for (float t = 0.0; t < num; t++) {
                    float weight = num - t;
                    vec2 usepos = pos + move * (t);
                    color += (texture2D(inputImageTexture, resettexCoord(usepos)) * weight);
                    total += weight;
                }
            }
            else if (motiontype == 2.0) {
                float b_scalex = motionlength.x;
                float b_scaley = motionlength.y;
                float b_roate = motionlength.z;
                for (float t = 0.0; t < num; t++) {
                    float weight = num - t;
                    vec2 usepos = pos;
                    float progress = t / 10.0;
                    if (b_scalex != 0.0 || b_scaley != 0.0) {
                        usepos = scalepos(pos, 1.0 + b_scalex * progress, 1.0 + b_scaley * progress);
                    }
                    if (b_roate != 0.0) {
                        usepos = roatepos(pos, roate + b_roate * progress);
                    }
                    color += (texture2D(inputImageTexture, resettexCoord(usepos)) * weight);
                    total += weight;
                }
            }
            gl_FragColor = color / total * alpha;
        }
        else {
            gl_FragColor = texture2D(inputImageTexture, pos) * alpha;
        }
    }
}

