//————————————————
//版权声明：本文为CSDN博主「点燃火柴」的原创文章，遵循CC 4.0 BY-SA版权协议，转载请附上原文出处链接及本声明。
//原文链接：https://blog.csdn.net/qw8704149/article/details/121412729
//        https://iquilezles.org/articles/distfunctions2d/

//测试形状渐变
precision highp float;
varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform lowp float progress;
uniform lowp float ratio;

float PI = 3.1415926;
float size_circle = 0.2;
float size_rect = 0.2;
vec2 center = vec2(0.5);


vec4 getToColor(vec2 uv) {
    return texture2D(inputImageTexture2, uv);
}
vec4 getFromColor(vec2 uv) {
    return texture2D(inputImageTexture, uv);
}
float dot2(vec2 v) { return dot(v, v); }

vec2 doroate(vec2 st, float roate) {
    float c = cos(roate);
    float s = sin(roate);
    float dx = st.x * c + st.y * s;
    float dy = -st.x * s + st.y * c;
    st.x = dx;
    st.y = dy;
    return st;
}

//圆形
float sdCircle(vec2 p, float r)
{
    return length(p) - r;// 到圆表面的距离
}

//矩形
float sdBox(in vec2 p, in vec2 b)
{
    vec2 d = abs(p) - b;
    return length(max(d, 0.0)) + min(max(d.x, d.y), 0.0);
}

//圆角矩形
float sdBoxRadius(in vec2 p, in vec2 b, float radius)
{
    vec2 d = abs(p) - b + radius;
    return length(max(d, 0.0)) + min(max(d.x, d.y), 0.0) - radius;
}

//通过端点和宽度绘制矩形
//第一个是屏幕坐标，第二个是矩形的起点，上边中心点，第三个是矩形的终点，下边中心点，第四个参数是矩形的宽度，
float sdOrientedBox(in vec2 p, in vec2 a, in vec2 b, float th)
{
    float l = length(b - a);
    vec2 d = (b - a) / l;
    vec2 q = (p - (a + b) * 0.5);
    q = mat2(d.x, -d.y, d.y, d.x) * q;
    q = abs(q) - vec2(l, th) * 0.5;
    return length(max(q, 0.0)) + min(max(q.x, q.y), 0.0);
}

//菱形
float ndot(vec2 a, vec2 b) { return a.x * b.x - a.y * b.y; }
float sdRhombus(in vec2 p, in vec2 b)
{
    p = abs(p);
    float h = clamp(ndot(b - 2.0 * p, b) / dot(b, b), -1.0, 1.0);
    float d = length(p - 0.5 * b * vec2(1.0 - h, 1.0 + h));
    return d * sign(p.x * b.y + p.y * b.x - b.x * b.y);
}
//三角形
float sdTriangle(in vec2 p, in vec2 p0, in vec2 p1, in vec2 p2)
{
    vec2 e0 = p1 - p0, e1 = p2 - p1, e2 = p0 - p2;
    vec2 v0 = p - p0, v1 = p - p1, v2 = p - p2;
    vec2 pq0 = v0 - e0 * clamp(dot(v0, e0) / dot(e0, e0), 0.0, 1.0);
    vec2 pq1 = v1 - e1 * clamp(dot(v1, e1) / dot(e1, e1), 0.0, 1.0);
    vec2 pq2 = v2 - e2 * clamp(dot(v2, e2) / dot(e2, e2), 0.0, 1.0);
    float s = sign(e0.x * e2.y - e0.y * e2.x);
    vec2 d = min(min(vec2(dot(pq0, pq0), s * (v0.x * e0.y - v0.y * e0.x)),
                     vec2(dot(pq1, pq1), s * (v1.x * e1.y - v1.y * e1.x))),
                 vec2(dot(pq2, pq2), s * (v2.x * e2.y - v2.y * e2.x)));
    return -sqrt(d.x) * sign(d.y);
}
//散射
float transition(vec2 p) {
    float speed = 5.0;
    float usepro = progress / 0.2;
    //    会有num*2个条
    float num = 1.5;
    float circPos = atan(p.x, p.y) + usepro * speed;
    float modPos = mod(circPos, PI / num);
    float signed = sign(usepro - modPos);

    return signed < 0.5 ? 1.0 : -1.0;

}

//五边形
float sdPentagon(in vec2 p, in float r)
{
    const vec3 k = vec3(0.809016994, 0.587785252, 0.726542528);
    p.x = abs(p.x);
    p -= 2.0 * min(dot(vec2(-k.x, k.y), p), 0.0) * vec2(-k.x, k.y);
    p -= 2.0 * min(dot(vec2(k.x, k.y), p), 0.0) * vec2(k.x, k.y);
    p -= vec2(clamp(p.x, -r * k.z, r * k.z), r);
    return length(p) * sign(p.y);
}
//六边形
float sdHexagon(in vec2 p, in float r)
{
    const vec3 k = vec3(-0.866025404, 0.5, 0.577350269);
    p = abs(p);
    p -= 2.0 * min(dot(k.xy, p), 0.0) * k.xy;
    p -= vec2(clamp(p.x, -k.z * r, k.z * r), r);
    return length(p) * sign(p.y);
}
//八边形
float sdOctogon(in vec2 p, in float r)
{
    const vec3 k = vec3(-0.9238795325, 0.3826834323, 0.4142135623);
    p = abs(p);
    p -= 2.0 * min(dot(vec2(k.x, k.y), p), 0.0) * vec2(k.x, k.y);
    p -= 2.0 * min(dot(vec2(-k.x, k.y), p), 0.0) * vec2(-k.x, k.y);
    p -= vec2(clamp(p.x, -k.z * r, k.z * r), r);
    return length(p) * sign(p.y);
}

//扇形
float sdPie(in vec2 p, in vec2 c, in float r)
{
    p.x = abs(p.x);
    p.y = 1.0 - p.y;
    float l = length(p) - r;
    float m = length(p - c * clamp(dot(p, c), 0.0, r)); // c=sin/cos of aperture
    return max(l, m * sign(c.y * p.x - c.x * p.y));
}

//
float sdTunnel(in vec2 p, in vec2 wh)
{
    p.x = abs(p.x);
    p.y = -p.y;
    vec2 q = p - wh;

    float d1 = dot2(vec2(max(q.x, 0.0), q.y));
    q.x = (p.y > 0.0) ? q.x : length(p) - wh.x;
    float d2 = dot2(vec2(q.x, max(q.y, 0.0)));
    float d = sqrt(min(d1, d2));

    return (max(q.x, q.y) < 0.0) ? -d : d;
}

float sdTunnel2(in vec2 p, in vec2 wh)
{
    float halfw = wh.x / 2.0;
    float b = sdOrientedBox(p, vec2(0.0, -wh.y), vec2(0.0, wh.y), wh.x);
    float c1 = sdCircle(vec2(p.x, p.y - wh.y), halfw);
    float c2 = sdCircle(vec2(p.x, p.y + wh.y), halfw);
    return min(b, c1 < c2 ? c1 : c2);
}

//这个是对称缩减
float sdPie2(vec2 p, float r, float pro)
{
    pro *= 3.14159;
    if (abs(atan(p.x, p.y)) > pro) {
        return length(p) - r;
    }
    return 4.0;
}
//这个是单方面缩减
float sdPie3(vec2 p, float r, float pro)
{
    float angle = atan(p.x, p.y);
    float normalizedAngle = (angle + PI) / (2.0 * PI);
    normalizedAngle = normalizedAngle - floor(normalizedAngle);
    if (normalizedAngle > pro) {
        return length(p) - r;
    }
    return 4.0;
}

float sdHeart(vec2 p)
{
    p.x = abs(p.x);
    p.y = 1.0 - p.y;
    p.y -= 0.6;
    if (p.y + p.x > 1.0)
    return sqrt(dot2(p - vec2(0.25, 0.75))) - sqrt(2.0) / 4.0;
    return sqrt(min(dot2(p - vec2(0.00, 1.00)),
                    dot2(p - 0.5 * max(p.x + p.y, 0.0)))) * sign(p.x - p.y);
}

//圆角
float opRound(float value, in float r)
{
    return value - r;
}

//中空，会以之前边为中心放大图像 r
float opOnion(float value, in float r)
{
    return abs(value) - r;
}

void main() {

    vec2 st = textureCoordinate - center;
    if (ratio > 1.) {
        st.y = st.y / ratio;
    } else {
        st.x = st.x * ratio;
    }
    //    st=doroate(st,progress*10.0*PI);


    //   第一个是屏幕坐标，
    //第二个参数为三角形左下角顶点坐标，第三个参数为三角形右下角顶点坐标第四个参数表示三角形顶部顶点坐标
    float scale_Triangle = sdTriangle(st, vec2(-0.2, -0.2), vec2(0.2, -0.2), vec2(0.0, 0.2));
    float scale_rect = sdBox(st, vec2(size_rect));
    float scale_Pentagon = sdPentagon(st, size_rect);
    float scale_Hexagon = sdHexagon(st, size_rect);
    float scale_Octogon = sdOctogon(st, size_rect);

    float scale_circle = sdCircle(st, size_circle);
    float scale_Rhombus = sdRhombus(st, vec2(0.2, 0.1));
    float scale_heart = sdHeart(st * 5.0);
    //  扇形第一个是屏幕坐标，第二个参数用来确定扇形的大小，由x分量和y分量可以确定极坐标下第一象限的角度，第三个参数为扇形圆的半径
    //    float scale_pie = sdPie(st, vec2(0.5, 1.0), size_circle);
    //  sdPie2,sdPie3; 根据 atan 来判断

    //    scale_rect=opOnion(scale_rect,progress);
    //    scale_Pentagon=opOnion(scale_Pentagon,0.1);

    float scale1, scale2;
    float scale_mix;
    if (progress <= 0.2) {
        float usepro = smoothstep(0.0, 0.2, progress);
        float s0 = sdTriangle(st, vec2(-0.2, -0.2) * usepro,
                              vec2(0.2, -0.2) * usepro,
                              vec2(0.0, 0.2) * usepro);
        float s1 = transition(st);
        scale_mix = s1 < 0.0 ? s1 : s0;
        //        scale_mix = s1;
    } else if (progress <= 0.4) {
        scale1 = scale_Triangle;
        scale2 = scale_rect;
        scale_mix = mix(scale1, scale2, smoothstep(0.2, 0.4, progress));
    } else if (progress <= 0.5) {
        scale1 = scale_rect;
        scale2 = scale_Pentagon;
        scale_mix = mix(scale1, scale2, smoothstep(0.4, 0.5, progress));
    } else if (progress <= 0.6) {
        scale1 = scale_Pentagon;
        scale2 = scale_Hexagon;
        scale_mix = mix(scale1, scale2, smoothstep(0.5, 0.6, progress));
    } else if (progress <= 0.7) {
        scale1 = scale_Hexagon;
        scale2 = scale_Octogon;
        scale_mix = mix(scale1, scale2, smoothstep(0.6, 0.7, progress));
    } else if (progress <= 0.8) {
        scale1 = scale_Octogon;
        scale2 = scale_circle;
        scale_mix = mix(scale1, scale2, smoothstep(0.7, 0.8, progress));
    } else {
        float usepro = smoothstep(0.82, 1.0, progress);
        scale_mix = sdPie3(st, size_circle, usepro);
    }

    //    float Tunnel2 = sdTunnel2(st, vec2(0.1, 0.2));
    //    scale_mix = Tunnel2;

    gl_FragColor = mix(getFromColor(textureCoordinate),
                       getToColor(textureCoordinate),
                       1.0 - step(0.0, scale_mix)//smoothstep(0.0,0.01,pct)
    );
}
