//https://www.shadertoy.com/view/XdKSRd

precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float progress;
uniform float ratio;

//https://www.shadertoy.com/view/7syXWz

//----------------------------------------------------------------------------------------
//  1 out, 2 in...
float hash12(vec2 p)
{
    vec3 p3  = fract(vec3(p.xyx) * .1031);
    p3 += dot(p3, p3.yzx + 33.33);
    return fract((p3.x + p3.y) * p3.z);
}



//----------------------------------------------------------------------------------------
///  2 out, 2 in...
vec2 hash22(vec2 p)
{
    vec3 p3 = fract(vec3(p.xyx) * vec3(.1031, .1030, .0973));
    p3 += dot(p3, p3.yzx+33.33);
    return fract((p3.xx+p3.yz)*p3.zy);

}

//----------------------------------------------------------------------------------------
///  2 out, 3 in...
vec2 hash23(vec3 p3)
{
    p3 = fract(p3 * vec3(.1031, .1030, .0973));
    p3 += dot(p3, p3.yzx+33.33);
    return fract((p3.xx+p3.yz)*p3.zy);
}

vec4 buffera(vec2 U)
{
    vec4 O;
    vec2 pw=vec2(1.0/1080.0);
    vec2 uv=textureCoordinate;
    vec3 o = vec3(1,0,-1);
    #define N(swizzle) texture2D(iChannel0, uv+pw*o.swizzle)
    vec4 v = N(yy);
    vec4[6] n ;
    n[0]=N(xy);
    n[1]=N(xx);
    n[2]=N(yx);
    n[3]=N(yz);
    n[4]=N(zz);
    n[5]=N(zy);
//    (N(xy), N(xx), N(yx), N(yz), N(zz), N(zy));
    bool receptive = v.x >= 1.0;
    for (int i = 0; i < 6; i++) {
        receptive = receptive || (n[i].x >= 1.0);
    }
    O = vec4(v.x, float(receptive), 0, 0);
    return O;
}

vec4 bufferb(vec2 U )
{
    vec4 O;
    vec2 R = iResolution.xy, pw = 1./R, uv = U/R;
    vec3 o = vec3(1,0,-1);
    #define N(swizzle) buffera(uv+pw*o.swizzle)
    vec4 v = N(yy);
//    vec4[6] n = vec4[6](N(xy), N(xx), N(yx), N(yz), N(zz), N(zy));
    vec4[6] n ;
    n[0]=N(xy);
    n[1]=N(xx);
    n[2]=N(yx);
    n[3]=N(yz);
    n[4]=N(zz);
    n[5]=N(zy);
    float change;
    change = (v.y == 1.0) ? GAMMA : -ALPHA*v.x;
    float nrs = 0.0;
    for (int i = 0; i < 6; i++){
        if(n[i].y == 0.0){
            nrs += n[i].x;
        }
    }
    change += (nrs/6.0)*ALPHA;
    O = vec4(v.x + change, 0,0,0);

    float time=progress*200.0;
//    float time=mod(iTime,20.0)*10.0;

    if (time < 10.0) {
        O = vec4(0.0);
        O.r=BETA + 0.1 * (hash12(U) - 0.5);
        if (floor(U) == vec2(0,0) || (hash22(U).x < 1e-3 && hash22(U).y < 1e-2))  O = vec4(1.1,0,0,0);
    }
    if (mod(time,50.0) <1.0 && hash23(vec3(U,time)).x < 1e-3 && hash23(vec3(U,time)).y < 1e-3) O.x++;
    return O;
}

float T(vec2 p){
   return bufferb(mat2(1,0,.5,1) * mat2(1,0,0,1.25) * (p) / (1.25 * textureCoordinate.xy)).x;
}

vec4 render(vec2 fragCoord)
{
    float cell  = T(fragCoord);
    float cellx = T(fragCoord + vec2(1,0));
    float celly = T(fragCoord + vec2(0,1));

    vec4 fragColor = 0.75 * texture2D(inputImageTexture, vec2((fragCoord - iResolution.xy/2.)/iResolution.y));
    fragColor = mix(fragColor, vec4(1), smoothstep(0., 1., cell));
    fragColor = mix(fragColor, vec4(0.25), smoothstep(1., 6., cell));

    float focus = 6. - clamp(cell, 0., 4.);
    vec2 grad = vec2(cellx, celly) - cell;
    vec2 uv = (fragCoord + 100. * grad) / iResolution.xy;
    vec3 ray = vec3(uv, -0.5) - vec3(0.5,0.1,0);
    fragColor = textureLod(iChannel2, ray, focus);
    fragColor = mix(fragColor, texture(iChannel3, ray), smoothstep(0., 0.9, cell) - smoothstep(0.9, 1., cell));
    fragColor += 0.2 * dot(grad, vec2(1));
    return fragColor;
}


void main( vec2 fragCoord )
{
    float AA=2.0;
    gl_FragColor = vec4(0);
    for (float i = 0; i < AA; i++) {
        for (float j = 0; j < AA; j++){
            gl_FragColor += render(fragCoord + vec2(i,j)/2.) / AA*AA;
        }
    }
}
