// Heartfelt - by Martijn Steinrucken aka BigWings - 2017
// Email:countfrolic@gmail.com Twitter:@The_ArtOfCode
// License Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.

// I revisited the rain effect I did for another shader. This one is better in multiple ways:
// 1. The glass gets foggy.
// 2. Drops cut trails in the fog on the glass.
// 3. The amount of rain is adjustable (with Mouse.y)

// To have full control over the rain, uncomment the HAS_HEART define

// A video of the effect can be found here:
// https://www.youtube.com/watch?v=uiF5Tlw22PI&feature=youtu.be

// Music - Alone In The Dark - Vadim Kiselev
// https://soundcloud.com/ahmed-gado-1/sad-piano-alone-in-the-dark
// Rain sounds:
// https://soundcloud.com/elirtmusic/sleeping-sound-rain-and-thunder-1-hours

//https://www.shadertoy.com/view/WsV3zm

precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float progress;
uniform float ratio;

#define pi 3.1415926535
#define e 2.718281828

float r21(vec2 p) {
    return fract(sin(dot(p.xy, vec2(12.9898, 78.233))) * 4558.5453);
}

vec2 r22(vec2 p) {
    float f = r21(p);
    return (fract(vec2(f, floor(f) + fract(fract(f) * 100.))) - .5);
}

float powx = (e / (.1 * sqrt(2. * pi)));


float NormalCurve(float Mean, float Dist) {
    float z = (3.3 - Mean) * .005 * pow(powx, -1. * (pow(Dist - Mean, 2.) / (2. * 0.01)));
    return z;
}

vec3 Ripple(float Mean, float Dist, vec2 vec) {
    return vec3(normalize(vec), NormalCurve(Mean * 3., Dist) - NormalCurve(Mean * 3.1, Dist + .1) + NormalCurve(Mean * 3., Dist + .5) * .25 - NormalCurve(Mean * 3., Dist + .7) * .25);
}

vec3 getpoint(vec2 vec) {
    float resFactor = 1.0 / ratio;
    float dropCount = 20.;
    vec2 uv = vec - vec2(.5);
    vec3 col=vec3(0.0);
    float time = progress / 1500.0;
    for (float i = 1.; i <= dropCount; i += 1.) {
        float t = time + i / dropCount;
        vec2 p = r22(vec2(i, floor(t)));
        col += Ripple(fract(t), length((uv - p) * vec2(ratio, 1.)) * 30., uv - p);
    }
    return col;
}


void main()
{
    vec2 uv = textureCoordinate;

    // Time varying pixel color
    vec3 displacement = getpoint(uv);
    vec3 col = texture2D(inputImageTexture, uv - displacement.xy * displacement.z).xyz;
    //vec3 col = vec3(displacement.z);
    //vec3 col = texture(iChannel2, uv).xyz;

    //col += texture(iChannel3, vec2(uv.x,0)).x;

    // Output to screen
    gl_FragColor = vec4(col, 1.0);
}