//https://www.shadertoy.com/view/XtKfRV

precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float progress;
uniform float ratio;


vec3 rgb2hsv(vec3 c)
{
    vec4 k = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, k.wz), vec4(c.gb, k.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c)
{
    vec4 k = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + k.xyz) * 6.0 - k.www);
    return c.z * mix(k.xxx, clamp(p - k.xxx, 0.0, 1.0), c.y);
}

float saturate(float x)
{
    return clamp(x, 0., 1.);
}
vec2 hash21(float p) //1 in 2 out hash function
{
    vec3 p3 = fract(vec3(p) * vec3(.1031, .1030, .0973));
    p3 += dot(p3, p3.yzx + 19.19);
    return fract((p3.xx + p3.yz) * p3.zy) * 2. - 1.;

}

vec3 drawflare(vec2 p, float intensity, float rnd, float speed, float id)
{
    float flarehueoffset = (1. / 32.) * id * 0.1;
    float lingrad = distance(vec2(0.), p); //linear radial gradient
    float expgrad = 1. / exp(lingrad * (fract(rnd) * 0.66 + 0.33)); //exponential radial gradient
    vec3 colgrad = hsv2rgb(vec3(fract((expgrad * 8.) + (speed) + flarehueoffset), pow(1. - abs(expgrad * 2. - 1.), 0.45), 20.0 * expgrad * intensity)); //rainbow spectrum effect
    float blades = length(p * sin(3.0 * atan(p.x, p.y))); //draw 6 blades
    float comp = pow(1. - saturate(blades), 12.); // sharpen effect
    comp += saturate(expgrad - 0.9) * 3.;
    comp = pow(comp * expgrad, 8. + (1. - intensity) * 5.); // compose and sharpen effect
    return vec3(comp) * colgrad; // apply color
}
vec2 rotateUV(vec2 uv, float rotation)
{
    return vec2(
        cos(rotation) * uv.x + sin(rotation) * uv.y,
        cos(rotation) * uv.y - sin(rotation) * uv.x
    );
}


vec4 color1()
{
    float speed = progress/1500.0;
    vec2 uv = textureCoordinate;
    vec2 p = uv * 2. - 1.;
    p.x *= ratio;
    vec4 fragColor = vec4(0.0);
    for (float i = 0.; i < 32.; ++i) { //draw flares
         float intensity = smoothstep(0.99, 1., saturate(sin(speed + i))); // sequential flare intensity animation
         intensity += smoothstep(0.994, 1., saturate(sin(speed + i + 0.3))) * 0.1;
         if (intensity > 0.0) {
             vec2 myrandomoffset = hash21(i * 3.21);
             myrandomoffset.x = myrandomoffset.x * ratio;
             vec2 prot = rotateUV(p + myrandomoffset, (speed * 0.5)); //apply offset & rotate uvs
             //prot = fract( (prot*0.5+0.5) + vec2(iTime*0.2,0.) ) * 2. - 1.; //kaleidascope effect
             float rnd = i * 1.337; //per flare random val
//             todo prot*4.0大小
             fragColor.xyz += drawflare(prot*4.0, intensity, rnd, speed, i); //draw
         }
    }
    return fragColor;
}
vec4  getscreencolor(vec4 base,vec4 overlay){
    vec4 one=vec4(1.0);
    vec4 rtn=one-(one-base)*(one-overlay);
    return rtn;
}

void main() {
    vec2 uv = textureCoordinate;
    vec3 color = color1().rgb;
    color *= 20.0;
    color = color / (1.0 + color); //Tonemapping
    vec4 color2=vec4(color,1.0);
    vec4 color1=texture2D(inputImageTexture,uv);

    gl_FragColor =getscreencolor(color1,color2);
}