/// 灵魂出窍滤镜
precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform lowp float Time;

//⼀次灵魂出窍的时⻓
const float duration = 0.7;
//透明度上限
const float maxAlpha = 0.4;
//放⼤图⽚上限
const float maxScale = 1.8;

void main (void) {

    //进度(0~1)
    float progress = mod(Time, duration) / duration; // 0~1

    //透明度(0-0.4)
    float alpha = maxAlpha * (1.0 - progress);

    //缩放⽐例(1.0 - 1.8)
    float scale = 1.0 + (maxScale - 1.0) * progress;

    //放⼤纹理坐标
    //将顶点坐标对应的纹理坐标的 x 值到纹理中点的距离，缩⼩⼀定的⽐例。这次我们是改变了纹理坐标，⽽保持顶点坐标不变，同样达到了拉伸的效果
    float weakX = 0.5 + (textureCoordinate.x - 0.5) / scale;
    float weakY = 0.5 + (textureCoordinate.y - 0.5) / scale;

    //获取放⼤的纹理坐标
    vec2 weakTextureCoords = vec2(weakX, weakY);

    //通过上⾯的计算，我们得到了两个纹理颜⾊值 weakMask 和 mask， weakMask 是在 mask 的基础上做了放⼤处理4//读取到放⼤后的纹理坐标的纹素的颜⾊值
    vec4 weakMask = texture2D(inputImageTexture, weakTextureCoords);

    //读取原始的纹理坐标对应的纹素的颜⾊值
    vec4 mask = texture2D(inputImageTexture, textureCoordinate);

    //在GLSL 实现颜⾊混合⽅程式.默认颜⾊混合⽅程式 = mask * (1.0 - alpha) + weakMask * alpha
    //参考OpenGL 第⼆节课中的颜⾊混合
    //计算最终的颜⾊赋值给⽚元着⾊器的内置变量: gl_FragColor
    gl_FragColor = mask * (1.0 - alpha) + weakMask * alpha;

}