

// License: MIT
// Author: rectalogic
// ported by gre from https://gist.github.com/rectalogic/b86b90161503a0023231

// Converted from https://github.com/rectalogic/rendermix-basic-effects/blob/master/assets/com/rendermix/CrossZoom/CrossZoom.frag
// Which is based on https://github.com/evanw/glfx.js/blob/master/src/filters/blur/zoomblur.js
// With additional easing functions from https://github.com/rectalogic/rendermix-basic-effects/blob/master/assets/com/rendermix/Easing/Easing.glsllib

// 6-3
// https://gl-transitions.com/editor/CrossZoom

precision highp float;
varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform  float progress;
uniform  float ratio;

const float PI = 3.1415926;
float rotations=1.0; // 旋转多少圈  1;
float scale=3.0; // 扩大的比例= 8;
const vec4 backColor= vec4(0.15, 0.15, 0.15, 1.0);

//uniform float strength; // = 0.4
float strength = 0.4;

vec4 getToColor(vec2 uv){
    return texture2D(inputImageTexture2, uv);
}
vec4 getFromColor(vec2 uv){
    return texture2D(inputImageTexture, uv);
}

float Linear_ease(in float begin, in float change, in float duration, in float time) {
    return change * time / duration + begin;
}

float Exponential_easeInOut(in float begin, in float change, in float duration, in float time) {
    if (time == 0.0)
    return begin;
    else if (time == duration)
    return begin + change;
    time = time / (duration / 2.0);
    if (time < 1.0)
    return change / 2.0 * pow(2.0, 10.0 * (time - 1.0)) + begin;
    return change / 2.0 * (-pow(2.0, -10.0 * (time - 1.0)) + 2.0) + begin;
}

float Sinusoidal_easeInOut(in float begin, in float change, in float duration, in float time) {
    return -change / 2.0 * (cos(PI * time / duration) - 1.0) + begin;
}

float rand (vec2 co) {
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

vec3 crossFade(in vec2 uv, in float dissolve) {
    return mix(getFromColor(uv).rgb, getToColor(uv).rgb, dissolve);
}
vec2 doroate(vec2 c2,float roate,float scale){
    //计算旋转角度
    vec2 uv2  =c2-0.5;
    if(ratio>1.0){
        uv2.x=uv2.x*ratio;
    }else{
        uv2.y=uv2.y/ratio;
    }
    float c = cos(roate);
    float s = sin(roate);
    float dx2 = uv2.x*c+uv2.y*s;
    float dy2 = -uv2.x*s+uv2.y*c;
    if(ratio>1.0){
        dx2=dx2/ratio;
    }else{
        dy2=dy2*ratio;
    }
    vec2 rtn=vec2(0.5+dx2/scale,0.5+dy2/scale);
    return rtn;
}

void main() {
    vec2 texCoord = textureCoordinate.xy / vec2(1.0).xy;

    float angle = 2.0 * PI * rotations * progress;
    float currentScale = mix(scale, 1.0, 2.0 * abs(progress - 0.5));

    texCoord = doroate(texCoord,angle,currentScale);

    if (texCoord.x < 0.0 || texCoord.x > 1.0 ||
    texCoord.y < 0.0 || texCoord.y > 1.0){
        gl_FragColor=backColor;
    }else{
        // Linear interpolate center across center half of the image
        vec2 center = vec2(0.5);
        float dissolve = Exponential_easeInOut(0.0, 1.0, 1.0, progress);

        // Mirrored sinusoidal loop. 0->strength then strength->0
        float strength = Sinusoidal_easeInOut(0.0, strength, 0.5, progress);

        vec3 color = vec3(0.0);
        float total = 0.0;
        vec2 toCenter = center - texCoord;

        /* randomize the lookup values to hide the fixed number of samples */
        float offset = rand(textureCoordinate);

        float num=10.0;
        for (float t = 0.0; t <= num; t++) {
            float percent = (t + offset) / num;
            float weight = 4.0 * (percent - percent * percent);
            color += crossFade(texCoord + toCenter * percent * strength, dissolve) * weight;
            total += weight;
        }
        gl_FragColor= vec4(color / total, 1.0);
    }



}
