// 6-6
// https://gl-transitions.com/editor/swap
precision mediump float;
varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform lowp float progress;

//参考 5-2
//uniform float reflection;//反射 = 0.4
//uniform float perspective;//透视 = 0.2
//uniform float depth;//深度 = 3.0
float reflection = 0.4;
float perspective = 0.2;
float depth = 3.0;

const vec4 black = vec4(0.0, 0.0, 0.0, 1.0);
const vec2 boundMin = vec2(0.0, 0.0);
const vec2 boundMax = vec2(1.0, 1.0);

bool inBounds (vec2 p) {
    return all(lessThan(boundMin, p)) && all(lessThan(p, boundMax));
}

//vec2 project (vec2 p) {
//    return p * vec2(1.0, -1.2) + vec2(0.0, -0.02);
//}

vec2 project (vec2 p) {
    vec2 rtn= p * vec2(1.0, (1.0-depth/100.)) ;
    rtn.y=2.-rtn.y;
    return rtn;
}

vec4 getToColor(vec2 uv){
    return texture2D(inputImageTexture2, uv);
}
vec4 getFromColor(vec2 uv){
    return texture2D(inputImageTexture, uv);
}

vec4 bgColor (vec2 p, vec2 pfr, vec2 pto) {
    vec4 c = black;
    pfr = project(pfr);
    if (inBounds(pfr)) {
        c += mix(black, getFromColor(pfr), reflection * mix(0.0, 1.0, pfr.y));
    }
    pto = project(pto);
    if (inBounds(pto)) {
        c += mix(black, getToColor(pto), reflection * mix(0.0,1.0, pto.y));
    }
    return c;
}


void main() {
    vec2 p=textureCoordinate;
    vec2 pfr, pto = vec2(-1.);

    float size = mix(1.0, depth, progress);
    float persp = perspective * progress;
    pfr = (p + vec2(-0.0, -0.5)) * vec2(size/(1.0-perspective*progress), size/(1.0-size*persp*p.x)) + vec2(0.0, 0.5);

    size = mix(1.0, depth, 1.-progress);
    persp = perspective * (1.-progress);
    pto = (p + vec2(-1.0, -0.5)) * vec2(size/(1.0-perspective*(1.0-progress)), size/(1.0-size*persp*(0.5-p.x))) + vec2(1.0, 0.5);

    if (progress < 0.5) {
        if (inBounds(pfr)) {
            gl_FragColor= getFromColor(pfr);
            return;
        }
        if (inBounds(pto)) {
            gl_FragColor= getToColor(pto);
            return;
        }
    }
    if (inBounds(pto)) {
        gl_FragColor= getToColor(pto);
        return;
    }
    if (inBounds(pfr)) {
        gl_FragColor= getFromColor(pfr);
        return;
    }
    gl_FragColor= bgColor(p, pfr, pto);
}
