precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;
uniform float progress;

uniform float ratio;
uniform float ratiovideo;

vec4 getToColor(vec2 uv) {
    return texture2D(inputImageTexture2, uv);
}
vec4 getFromColor(vec2 uv) {
    return texture2D(inputImageTexture, uv);
}

vec3 rgbToHsv(vec3 c) {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsvToRgb(vec3 c) {//convertHsvToRgb
                       vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
                       vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
                       return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

vec3 addmax = vec3(0.0, 0.0, 0.2);
void main()
{

    vec2 texCoord = textureCoordinate.xy / vec2(1.0);
    vec4 color = progress <= 0.5 ?
    texture2D(inputImageTexture, textureCoordinate) :
    texture2D(inputImageTexture2, textureCoordinate);
    if (progress > 0.2 && progress < 0.8) {
        float mixvalue = 1.0 - abs(progress - 0.5) / 0.3;
        vec3 hsl = rgbToHsv(color.rgb);
        hsl += addmax * mixvalue;
        color.rgb = hsvToRgb(hsl);
        color = min(color, vec4(1.0));
    }

    float ratio2;
    bool change=false;
    if (ratio >= 1.0 && ratiovideo >= 1.0 || ratio <= 1.0 && ratiovideo <= 1.0) {
        ratio2 = ratiovideo;
    } else {
        ratio2 = 1.0 / ratiovideo;
        change=true;
    }

    float scale = ratio2 / ratio;
    if (scale < 1.0) {
        texCoord.y = 0.5 + (texCoord.y - 0.5) * scale;
    } else {
        texCoord.x = 0.5 + (texCoord.x - 0.5) / scale;
    }
    if(change){
        texCoord = texCoord.yx;
    }

    vec4 textureColor2 = texture2D(inputImageTexture3, texCoord);
    mediump vec4 whiteColor = vec4(1.0);
    lowp vec4 textureColor3 = whiteColor - ((whiteColor - textureColor2) * (whiteColor - color));
    gl_FragColor = textureColor3;
}
